/*      */ package org.hibernate.cfg.annotations;
/*      */ 
/*      */ import java.util.ArrayList;
/*      */ import java.util.Comparator;
/*      */ import java.util.HashMap;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.Set;
/*      */ import java.util.SortedMap;
/*      */ import java.util.SortedSet;
/*      */ import java.util.StringTokenizer;
/*      */ import javax.persistence.AttributeOverride;
/*      */ import javax.persistence.AttributeOverrides;
/*      */ import javax.persistence.Embeddable;
/*      */ import javax.persistence.FetchType;
/*      */ import javax.persistence.JoinTable;
/*      */ import javax.persistence.ManyToMany;
/*      */ import org.apache.commons.logging.Log;
/*      */ import org.apache.commons.logging.LogFactory;
/*      */ import org.hibernate.AnnotationException;
/*      */ import org.hibernate.AssertionFailure;
/*      */ import org.hibernate.MappingException;
/*      */ import org.hibernate.annotations.BatchSize;
/*      */ import org.hibernate.annotations.Cache;
/*      */ import org.hibernate.annotations.CollectionId;
/*      */ import org.hibernate.annotations.CollectionOfElements;
/*      */ import org.hibernate.annotations.Fetch;
/*      */ import org.hibernate.annotations.Filter;
/*      */ import org.hibernate.annotations.FilterJoinTable;
/*      */ import org.hibernate.annotations.FilterJoinTables;
/*      */ import org.hibernate.annotations.Filters;
/*      */ import org.hibernate.annotations.ForeignKey;
/*      */ import org.hibernate.annotations.Immutable;
/*      */ import org.hibernate.annotations.LazyCollection;
/*      */ import org.hibernate.annotations.LazyCollectionOption;
/*      */ import org.hibernate.annotations.Loader;
/*      */ import org.hibernate.annotations.ManyToAny;
/*      */ import org.hibernate.annotations.OptimisticLock;
/*      */ import org.hibernate.annotations.Persister;
/*      */ import org.hibernate.annotations.ResultCheckStyle;
/*      */ import org.hibernate.annotations.SQLDelete;
/*      */ import org.hibernate.annotations.SQLDeleteAll;
/*      */ import org.hibernate.annotations.SQLInsert;
/*      */ import org.hibernate.annotations.SQLUpdate;
/*      */ import org.hibernate.annotations.Sort;
/*      */ import org.hibernate.annotations.SortType;
/*      */ import org.hibernate.annotations.Where;
/*      */ import org.hibernate.annotations.WhereJoinTable;
/*      */ import org.hibernate.annotations.common.reflection.XClass;
/*      */ import org.hibernate.annotations.common.reflection.XProperty;
/*      */ import org.hibernate.cfg.AnnotatedClassType;
/*      */ import org.hibernate.cfg.AnnotationBinder;
/*      */ import org.hibernate.cfg.BinderHelper;
/*      */ import org.hibernate.cfg.CollectionSecondPass;
/*      */ import org.hibernate.cfg.Ejb3Column;
/*      */ import org.hibernate.cfg.Ejb3JoinColumn;
/*      */ import org.hibernate.cfg.ExtendedMappings;
/*      */ import org.hibernate.cfg.IndexColumn;
/*      */ import org.hibernate.cfg.Mappings;
/*      */ import org.hibernate.cfg.PropertyData;
/*      */ import org.hibernate.cfg.PropertyHolder;
/*      */ import org.hibernate.cfg.PropertyHolderBuilder;
/*      */ import org.hibernate.cfg.PropertyInferredData;
/*      */ import org.hibernate.cfg.PropertyPreloadedData;
/*      */ import org.hibernate.cfg.SecondPass;
/*      */ import org.hibernate.engine.ExecuteUpdateResultCheckStyle;
/*      */ import org.hibernate.engine.FilterDefinition;
/*      */ import org.hibernate.mapping.Any;
/*      */ import org.hibernate.mapping.Backref;
/*      */ import org.hibernate.mapping.Column;
/*      */ import org.hibernate.mapping.Component;
/*      */ import org.hibernate.mapping.DependantValue;
/*      */ import org.hibernate.mapping.IdGenerator;
/*      */ import org.hibernate.mapping.Join;
/*      */ import org.hibernate.mapping.KeyValue;
/*      */ import org.hibernate.mapping.ManyToOne;
/*      */ import org.hibernate.mapping.PersistentClass;
/*      */ import org.hibernate.mapping.Property;
/*      */ import org.hibernate.mapping.Selectable;
/*      */ import org.hibernate.mapping.SimpleValue;
/*      */ import org.hibernate.mapping.Table;
/*      */ import org.hibernate.mapping.Value;
/*      */ import org.hibernate.util.StringHelper;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public abstract class CollectionBinder
/*      */ {
/*   93 */   private static final Log log = LogFactory.getLog(CollectionBinder.class);
/*      */   
/*      */   protected org.hibernate.mapping.Collection collection;
/*      */   protected String propertyName;
/*      */   PropertyHolder propertyHolder;
/*      */   int batchSize;
/*      */   private String mappedBy;
/*      */   private XClass collectionType;
/*      */   private XClass targetEntity;
/*      */   private ExtendedMappings mappings;
/*      */   private Ejb3JoinColumn[] inverseJoinColumns;
/*      */   private String cascadeStrategy;
/*      */   String cacheConcurrencyStrategy;
/*      */   String cacheRegionName;
/*      */   private boolean oneToMany;
/*      */   protected IndexColumn indexColumn;
/*      */   private String orderBy;
/*      */   protected String hqlOrderBy;
/*      */   private boolean isSorted;
/*      */   private Class comparator;
/*      */   private boolean hasToBeSorted;
/*      */   protected boolean cascadeDeleteEnabled;
/*      */   protected String mapKeyPropertyName;
/*  116 */   private boolean insertable = true;
/*  117 */   private boolean updatable = true;
/*      */   
/*      */   private Ejb3JoinColumn[] fkJoinColumns;
/*      */   
/*      */   private boolean isExplicitAssociationTable;
/*      */   
/*      */   private Ejb3Column[] elementColumns;
/*      */   private boolean isEmbedded;
/*      */   private XProperty property;
/*      */   private boolean ignoreNotFound;
/*      */   
/*      */   public void setUpdatable(boolean updatable)
/*      */   {
/*  130 */     this.updatable = updatable;
/*      */   }
/*      */   
/*      */   public void setInsertable(boolean insertable) {
/*  134 */     this.insertable = insertable;
/*      */   }
/*      */   
/*      */   public void setCascadeStrategy(String cascadeStrategy)
/*      */   {
/*  139 */     this.cascadeStrategy = cascadeStrategy;
/*      */   }
/*      */   
/*      */   public void setPropertyAccessorName(String propertyAccessorName) {
/*  143 */     this.propertyAccessorName = propertyAccessorName;
/*      */   }
/*      */   
/*      */ 
/*      */   public void setInverseJoinColumns(Ejb3JoinColumn[] inverseJoinColumns)
/*      */   {
/*  149 */     this.inverseJoinColumns = inverseJoinColumns;
/*      */   }
/*      */   
/*      */   public void setJoinColumns(Ejb3JoinColumn[] joinColumns) {
/*  153 */     this.joinColumns = joinColumns;
/*      */   }
/*      */   
/*      */ 
/*      */   public void setPropertyHolder(PropertyHolder propertyHolder)
/*      */   {
/*  159 */     this.propertyHolder = propertyHolder;
/*      */   }
/*      */   
/*      */   public void setBatchSize(BatchSize batchSize) {
/*  163 */     this.batchSize = (batchSize == null ? -1 : batchSize.size());
/*      */   }
/*      */   
/*      */   public void setEjb3OrderBy(javax.persistence.OrderBy orderByAnn) {
/*  167 */     if (orderByAnn != null) {
/*  168 */       this.hqlOrderBy = orderByAnn.value();
/*      */     }
/*      */   }
/*      */   
/*      */   public void setSqlOrderBy(org.hibernate.annotations.OrderBy orderByAnn) {
/*  173 */     if ((orderByAnn != null) && 
/*  174 */       (!BinderHelper.isDefault(orderByAnn.clause()))) this.orderBy = orderByAnn.clause();
/*      */   }
/*      */   
/*      */   public void setSort(Sort sortAnn)
/*      */   {
/*  179 */     if (sortAnn != null) {
/*  180 */       this.isSorted = (!SortType.UNSORTED.equals(sortAnn.type()));
/*  181 */       if ((this.isSorted) && (SortType.COMPARATOR.equals(sortAnn.type()))) {
/*  182 */         this.comparator = sortAnn.comparator();
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static CollectionBinder getCollectionBinder(String entityName, XProperty property, boolean isIndexed)
/*      */   {
/*  194 */     if (property.isArray()) {
/*  195 */       if (property.getElementClass().isPrimitive()) {
/*  196 */         return new PrimitiveArrayBinder();
/*      */       }
/*      */       
/*  199 */       return new ArrayBinder();
/*      */     }
/*      */     
/*  202 */     if (property.isCollection())
/*      */     {
/*  204 */       Class returnedClass = property.getCollectionClass();
/*  205 */       if (Set.class.equals(returnedClass)) {
/*  206 */         if (property.isAnnotationPresent(CollectionId.class)) {
/*  207 */           throw new AnnotationException("Set do not support @CollectionId: " + StringHelper.qualify(entityName, property.getName()));
/*      */         }
/*      */         
/*  210 */         return new SetBinder();
/*      */       }
/*  212 */       if (SortedSet.class.equals(returnedClass)) {
/*  213 */         if (property.isAnnotationPresent(CollectionId.class)) {
/*  214 */           throw new AnnotationException("Set do not support @CollectionId: " + StringHelper.qualify(entityName, property.getName()));
/*      */         }
/*      */         
/*  217 */         return new SetBinder(true);
/*      */       }
/*  219 */       if (Map.class.equals(returnedClass)) {
/*  220 */         if (property.isAnnotationPresent(CollectionId.class)) {
/*  221 */           throw new AnnotationException("Map do not support @CollectionId: " + StringHelper.qualify(entityName, property.getName()));
/*      */         }
/*      */         
/*  224 */         return new MapBinder();
/*      */       }
/*  226 */       if (SortedMap.class.equals(returnedClass)) {
/*  227 */         if (property.isAnnotationPresent(CollectionId.class)) {
/*  228 */           throw new AnnotationException("Map do not support @CollectionId: " + StringHelper.qualify(entityName, property.getName()));
/*      */         }
/*      */         
/*  231 */         return new MapBinder(true);
/*      */       }
/*  233 */       if (java.util.Collection.class.equals(returnedClass)) {
/*  234 */         if (property.isAnnotationPresent(CollectionId.class)) {
/*  235 */           return new IdBagBinder();
/*      */         }
/*      */         
/*  238 */         return new BagBinder();
/*      */       }
/*      */       
/*  241 */       if (List.class.equals(returnedClass)) {
/*  242 */         if (isIndexed) {
/*  243 */           if (property.isAnnotationPresent(CollectionId.class)) {
/*  244 */             throw new AnnotationException("List do not support @CollectionId and @IndexColumn at the same time: " + StringHelper.qualify(entityName, property.getName()));
/*      */           }
/*      */           
/*  247 */           return new ListBinder();
/*      */         }
/*  249 */         if (property.isAnnotationPresent(CollectionId.class)) {
/*  250 */           return new IdBagBinder();
/*      */         }
/*      */         
/*  253 */         return new BagBinder();
/*      */       }
/*      */       
/*      */ 
/*  257 */       throw new AnnotationException(returnedClass.getName() + " collection not yet supported: " + StringHelper.qualify(entityName, property.getName()));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  264 */     throw new AnnotationException("Illegal attempt to map a non collection as a @OneToMany, @ManyToMany or @CollectionOfElements: " + StringHelper.qualify(entityName, property.getName()));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   protected CollectionBinder() {}
/*      */   
/*      */ 
/*      */ 
/*      */   protected CollectionBinder(boolean sorted)
/*      */   {
/*  275 */     this.hasToBeSorted = sorted;
/*      */   }
/*      */   
/*      */   public void setMappedBy(String mappedBy) {
/*  279 */     this.mappedBy = mappedBy;
/*      */   }
/*      */   
/*      */   public void setTableBinder(TableBinder tableBinder) {
/*  283 */     this.tableBinder = tableBinder;
/*      */   }
/*      */   
/*      */   public void setCollectionType(XClass collectionType) {
/*  287 */     this.collectionType = collectionType;
/*      */   }
/*      */   
/*      */   public void setTargetEntity(XClass targetEntity) {
/*  291 */     this.targetEntity = targetEntity;
/*      */   }
/*      */   
/*      */   public void setMappings(ExtendedMappings mappings) {
/*  295 */     this.mappings = mappings;
/*      */   }
/*      */   
/*      */   protected abstract org.hibernate.mapping.Collection createCollection(PersistentClass paramPersistentClass);
/*      */   
/*      */   public org.hibernate.mapping.Collection getCollection() {
/*  301 */     return this.collection;
/*      */   }
/*      */   
/*      */   public void setPropertyName(String propertyName) {
/*  305 */     this.propertyName = propertyName;
/*      */   }
/*      */   
/*      */   public void bind() {
/*  309 */     this.collection = createCollection(this.propertyHolder.getPersistentClass());
/*  310 */     log.debug("Collection role: " + StringHelper.qualify(this.propertyHolder.getPath(), this.propertyName));
/*  311 */     this.collection.setRole(StringHelper.qualify(this.propertyHolder.getPath(), this.propertyName));
/*  312 */     this.collection.setNodeName(this.propertyName);
/*      */     
/*  314 */     if ((this.property.isAnnotationPresent(org.hibernate.annotations.MapKey.class)) && (this.mapKeyPropertyName != null)) {
/*  315 */       throw new AnnotationException("Cannot mix @javax.persistence.MapKey and @org.hibernate.annotations.MapKey on the same collection: " + StringHelper.qualify(this.propertyHolder.getPath(), this.propertyName));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  324 */     defineFetchingStrategy();
/*  325 */     this.collection.setBatchSize(this.batchSize);
/*  326 */     if ((this.orderBy != null) && (this.hqlOrderBy != null)) {
/*  327 */       throw new AnnotationException("Cannot use sql order by clause in conjunction of EJB3 order by clause: " + safeCollectionRole());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  332 */     this.collection.setMutable(!this.property.isAnnotationPresent(Immutable.class));
/*  333 */     OptimisticLock lockAnn = (OptimisticLock)this.property.getAnnotation(OptimisticLock.class);
/*  334 */     if (lockAnn != null) this.collection.setOptimisticLocked(!lockAnn.excluded());
/*  335 */     Persister persisterAnn = (Persister)this.property.getAnnotation(Persister.class);
/*  336 */     if (persisterAnn != null) { this.collection.setCollectionPersisterClass(persisterAnn.impl());
/*      */     }
/*      */     
/*  339 */     if (this.orderBy != null) this.collection.setOrderBy(this.orderBy);
/*  340 */     if (this.isSorted) {
/*  341 */       this.collection.setSorted(true);
/*  342 */       if (this.comparator != null) {
/*      */         try {
/*  344 */           this.collection.setComparator((Comparator)this.comparator.newInstance());
/*      */         }
/*      */         catch (ClassCastException e) {
/*  347 */           throw new AnnotationException("Comparator not implementing java.util.Comparator class: " + this.comparator.getName() + "(" + safeCollectionRole() + ")");
/*      */ 
/*      */         }
/*      */         catch (Exception e)
/*      */         {
/*      */ 
/*  353 */           throw new AnnotationException("Could not instantiate comparator class: " + this.comparator.getName() + "(" + safeCollectionRole() + ")");
/*      */ 
/*      */         }
/*      */         
/*      */       }
/*      */       
/*      */ 
/*      */     }
/*  361 */     else if (this.hasToBeSorted) {
/*  362 */       throw new AnnotationException("A sorted collection has to define @Sort: " + safeCollectionRole());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  370 */     if (StringHelper.isNotEmpty(this.cacheConcurrencyStrategy)) {
/*  371 */       this.collection.setCacheConcurrencyStrategy(this.cacheConcurrencyStrategy);
/*  372 */       this.collection.setCacheRegionName(this.cacheRegionName);
/*      */     }
/*      */     
/*      */ 
/*  376 */     SQLInsert sqlInsert = (SQLInsert)this.property.getAnnotation(SQLInsert.class);
/*  377 */     SQLUpdate sqlUpdate = (SQLUpdate)this.property.getAnnotation(SQLUpdate.class);
/*  378 */     SQLDelete sqlDelete = (SQLDelete)this.property.getAnnotation(SQLDelete.class);
/*  379 */     SQLDeleteAll sqlDeleteAll = (SQLDeleteAll)this.property.getAnnotation(SQLDeleteAll.class);
/*  380 */     Loader loader = (Loader)this.property.getAnnotation(Loader.class);
/*  381 */     if (sqlInsert != null) {
/*  382 */       this.collection.setCustomSQLInsert(sqlInsert.sql().trim(), sqlInsert.callable(), ExecuteUpdateResultCheckStyle.parse(sqlInsert.check().toString().toLowerCase()));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  387 */     if (sqlUpdate != null) {
/*  388 */       this.collection.setCustomSQLUpdate(sqlUpdate.sql(), sqlUpdate.callable(), ExecuteUpdateResultCheckStyle.parse(sqlUpdate.check().toString().toLowerCase()));
/*      */     }
/*      */     
/*      */ 
/*  392 */     if (sqlDelete != null) {
/*  393 */       this.collection.setCustomSQLDelete(sqlDelete.sql(), sqlDelete.callable(), ExecuteUpdateResultCheckStyle.parse(sqlDelete.check().toString().toLowerCase()));
/*      */     }
/*      */     
/*      */ 
/*  397 */     if (sqlDeleteAll != null) {
/*  398 */       this.collection.setCustomSQLDeleteAll(sqlDeleteAll.sql(), sqlDeleteAll.callable(), ExecuteUpdateResultCheckStyle.parse(sqlDeleteAll.check().toString().toLowerCase()));
/*      */     }
/*      */     
/*      */ 
/*  402 */     if (loader != null) {
/*  403 */       this.collection.setLoaderName(loader.namedQuery());
/*      */     }
/*      */     
/*      */ 
/*  407 */     boolean isMappedBy = !BinderHelper.isDefault(this.mappedBy);
/*  408 */     this.collection.setInverse(isMappedBy);
/*      */     
/*      */ 
/*  411 */     if ((!this.oneToMany) && (isMappedBy)) {
/*  412 */       this.mappings.addMappedBy(getCollectionType().getName(), this.mappedBy, this.propertyName);
/*      */     }
/*      */     
/*  415 */     XClass collectionType = getCollectionType();
/*  416 */     SecondPass sp = getSecondPass(this.fkJoinColumns, this.joinColumns, this.inverseJoinColumns, this.elementColumns, this.mapKeyColumns, this.mapKeyManyToManyColumns, this.isEmbedded, this.property, collectionType, this.ignoreNotFound, this.oneToMany, this.tableBinder, this.mappings);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  426 */     if ((collectionType.isAnnotationPresent(Embeddable.class)) || (this.property.isAnnotationPresent(CollectionOfElements.class)))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*  431 */       this.mappings.addSecondPass(sp, !isMappedBy);
/*      */     }
/*      */     else {
/*  434 */       this.mappings.addSecondPass(sp, !isMappedBy);
/*      */     }
/*      */     
/*  437 */     this.mappings.addCollection(this.collection);
/*      */     
/*      */ 
/*  440 */     PropertyBinder binder = new PropertyBinder();
/*  441 */     binder.setName(this.propertyName);
/*  442 */     binder.setValue(this.collection);
/*  443 */     binder.setCascade(this.cascadeStrategy);
/*  444 */     if ((this.cascadeStrategy != null) && (this.cascadeStrategy.indexOf("delete-orphan") >= 0)) {
/*  445 */       this.collection.setOrphanDelete(true);
/*      */     }
/*  447 */     binder.setPropertyAccessorName(this.propertyAccessorName);
/*  448 */     binder.setProperty(this.property);
/*  449 */     binder.setInsertable(this.insertable);
/*  450 */     binder.setUpdatable(this.updatable);
/*  451 */     Property prop = binder.make();
/*      */     
/*  453 */     this.propertyHolder.addProperty(prop);
/*      */   }
/*      */   
/*      */   private void defineFetchingStrategy() {
/*  457 */     LazyCollection lazy = (LazyCollection)this.property.getAnnotation(LazyCollection.class);
/*  458 */     Fetch fetch = (Fetch)this.property.getAnnotation(Fetch.class);
/*  459 */     javax.persistence.OneToMany oneToMany = (javax.persistence.OneToMany)this.property.getAnnotation(javax.persistence.OneToMany.class);
/*  460 */     ManyToMany manyToMany = (ManyToMany)this.property.getAnnotation(ManyToMany.class);
/*  461 */     CollectionOfElements elements = (CollectionOfElements)this.property.getAnnotation(CollectionOfElements.class);
/*  462 */     ManyToAny manyToAny = (ManyToAny)this.property.getAnnotation(ManyToAny.class);
/*      */     FetchType fetchType;
/*  464 */     if (oneToMany != null) {
/*  465 */       fetchType = oneToMany.fetch();
/*      */     } else { FetchType fetchType;
/*  467 */       if (manyToMany != null) {
/*  468 */         fetchType = manyToMany.fetch();
/*      */       } else { FetchType fetchType;
/*  470 */         if (elements != null) {
/*  471 */           fetchType = elements.fetch();
/*      */         } else { FetchType fetchType;
/*  473 */           if (manyToAny != null) {
/*  474 */             fetchType = FetchType.LAZY;
/*      */           }
/*      */           else
/*  477 */             throw new AssertionFailure("Define fetch strategy on a property not annotated with @ManyToOne nor @OneToMany nor @CollectionOfElements");
/*      */         }
/*      */       } }
/*      */     FetchType fetchType;
/*  481 */     if (lazy != null) {
/*  482 */       this.collection.setLazy(lazy.value() != LazyCollectionOption.FALSE);
/*  483 */       this.collection.setExtraLazy(lazy.value() == LazyCollectionOption.EXTRA);
/*      */     }
/*      */     else {
/*  486 */       this.collection.setLazy(fetchType == FetchType.LAZY);
/*  487 */       this.collection.setExtraLazy(false);
/*      */     }
/*  489 */     if (fetch != null) {
/*  490 */       if (fetch.value() == org.hibernate.annotations.FetchMode.JOIN) {
/*  491 */         this.collection.setFetchMode(org.hibernate.FetchMode.JOIN);
/*  492 */         this.collection.setLazy(false);
/*      */       }
/*  494 */       else if (fetch.value() == org.hibernate.annotations.FetchMode.SELECT) {
/*  495 */         this.collection.setFetchMode(org.hibernate.FetchMode.SELECT);
/*      */       }
/*  497 */       else if (fetch.value() == org.hibernate.annotations.FetchMode.SUBSELECT) {
/*  498 */         this.collection.setFetchMode(org.hibernate.FetchMode.SELECT);
/*  499 */         this.collection.setSubselectLoadable(true);
/*  500 */         this.collection.getOwner().setSubselectLoadableCollections(true);
/*      */       }
/*      */       else {
/*  503 */         throw new AssertionFailure("Unknown FetchMode: " + fetch.value());
/*      */       }
/*      */     }
/*      */     else {
/*  507 */       this.collection.setFetchMode(AnnotationBinder.getFetchMode(fetchType));
/*      */     }
/*      */   }
/*      */   
/*      */   private XClass getCollectionType() {
/*  512 */     if (AnnotationBinder.isDefault(this.targetEntity, this.mappings)) {
/*  513 */       if (this.collectionType != null) {
/*  514 */         return this.collectionType;
/*      */       }
/*      */       
/*  517 */       String errorMsg = "Collection has neither generic type or OneToMany.targetEntity() defined: " + safeCollectionRole();
/*      */       
/*  519 */       throw new AnnotationException(errorMsg);
/*      */     }
/*      */     
/*      */ 
/*  523 */     return this.targetEntity;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public SecondPass getSecondPass(final Ejb3JoinColumn[] fkJoinColumns, final Ejb3JoinColumn[] keyColumns, final Ejb3JoinColumn[] inverseColumns, final Ejb3Column[] elementColumns, Ejb3Column[] mapKeyColumns, Ejb3JoinColumn[] mapKeyManyToManyColumns, final boolean isEmbedded, final XProperty property, final XClass collType, final boolean ignoreNotFound, final boolean unique, final TableBinder assocTableBinder, final ExtendedMappings mappings)
/*      */   {
/*  537 */     new CollectionSecondPass(mappings, this.collection)
/*      */     {
/*      */       public void secondPass(Map persistentClasses, Map inheritedMetas) throws MappingException
/*      */       {
/*  541 */         CollectionBinder.this.bindStarToManySecondPass(persistentClasses, collType, fkJoinColumns, keyColumns, inverseColumns, elementColumns, isEmbedded, property, unique, assocTableBinder, ignoreNotFound, mappings);
/*      */       }
/*      */     };
/*      */   }
/*      */   
/*      */ 
/*      */   private TableBinder tableBinder;
/*      */   
/*      */   private Ejb3Column[] mapKeyColumns;
/*      */   
/*      */   private Ejb3JoinColumn[] mapKeyManyToManyColumns;
/*      */   
/*      */   protected HashMap<String, IdGenerator> localGenerators;
/*      */   
/*      */   private String propertyAccessorName;
/*      */   
/*      */   private Ejb3JoinColumn[] joinColumns;
/*      */   protected boolean bindStarToManySecondPass(Map persistentClasses, XClass collType, Ejb3JoinColumn[] fkJoinColumns, Ejb3JoinColumn[] keyColumns, Ejb3JoinColumn[] inverseColumns, Ejb3Column[] elementColumns, boolean isEmbedded, XProperty property, boolean unique, TableBinder associationTableBinder, boolean ignoreNotFound, ExtendedMappings mappings)
/*      */   {
/*  560 */     PersistentClass persistentClass = (PersistentClass)persistentClasses.get(collType.getName());
/*  561 */     boolean reversePropertyInJoin = false;
/*  562 */     if ((persistentClass != null) && (StringHelper.isNotEmpty(this.mappedBy))) {
/*      */       try {
/*  564 */         reversePropertyInJoin = 0 != persistentClass.getJoinNumber(persistentClass.getRecursiveProperty(this.mappedBy));
/*      */ 
/*      */       }
/*      */       catch (MappingException e)
/*      */       {
/*  569 */         StringBuilder error = new StringBuilder(80);
/*  570 */         error.append("mappedBy reference an unknown target entity property: ").append(collType).append(".").append(this.mappedBy).append(" in ").append(this.collection.getOwnerEntityName()).append(".").append(property.getName());
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  576 */         throw new AnnotationException(error.toString());
/*      */       }
/*      */     }
/*  579 */     if ((persistentClass != null) && (!reversePropertyInJoin) && (this.oneToMany) && (!this.isExplicitAssociationTable) && (((this.joinColumns[0].isImplicit()) && (!BinderHelper.isDefault(this.mappedBy))) || (!fkJoinColumns[0].isImplicit())))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  587 */       bindOneToManySecondPass(getCollection(), persistentClasses, fkJoinColumns, collType, this.cascadeDeleteEnabled, ignoreNotFound, this.hqlOrderBy, mappings);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  596 */       return true;
/*      */     }
/*      */     
/*      */ 
/*  600 */     bindManyToManySecondPass(this.collection, persistentClasses, keyColumns, inverseColumns, elementColumns, isEmbedded, collType, ignoreNotFound, unique, this.cascadeDeleteEnabled, associationTableBinder, property, this.propertyHolder, this.hqlOrderBy, mappings);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  611 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void bindOneToManySecondPass(org.hibernate.mapping.Collection collection, Map persistentClasses, Ejb3JoinColumn[] fkJoinColumns, XClass collectionType, boolean cascadeDeleteEnabled, boolean ignoreNotFound, String hqlOrderBy, ExtendedMappings extendedMappings)
/*      */   {
/*  620 */     if (log.isDebugEnabled()) {
/*  621 */       log.debug("Binding a OneToMany: " + this.propertyHolder.getEntityName() + "." + this.propertyName + " through a foreign key");
/*      */     }
/*      */     
/*      */ 
/*  625 */     org.hibernate.mapping.OneToMany oneToMany = new org.hibernate.mapping.OneToMany(collection.getOwner());
/*  626 */     collection.setElement(oneToMany);
/*  627 */     oneToMany.setReferencedEntityName(collectionType.getName());
/*  628 */     oneToMany.setIgnoreNotFound(ignoreNotFound);
/*      */     
/*  630 */     String assocClass = oneToMany.getReferencedEntityName();
/*  631 */     PersistentClass associatedClass = (PersistentClass)persistentClasses.get(assocClass);
/*  632 */     String orderBy = buildOrderByClauseFromHql(hqlOrderBy, associatedClass, collection.getRole());
/*  633 */     if (orderBy != null) collection.setOrderBy(orderBy);
/*  634 */     if (this.mappings == null) {
/*  635 */       throw new AssertionFailure("CollectionSecondPass for oneToMany should not be called with null mappings");
/*      */     }
/*      */     
/*      */ 
/*  639 */     Map<String, Join> joins = this.mappings.getJoins(assocClass);
/*  640 */     if (associatedClass == null) {
/*  641 */       throw new MappingException("Association references unmapped class: " + assocClass);
/*      */     }
/*      */     
/*      */ 
/*  645 */     oneToMany.setAssociatedClass(associatedClass);
/*  646 */     for (Ejb3JoinColumn column : fkJoinColumns) {
/*  647 */       column.setPersistentClass(associatedClass, joins);
/*  648 */       column.setJoins(joins);
/*  649 */       collection.setCollectionTable(column.getTable());
/*      */     }
/*  651 */     log.info("Mapping collection: " + collection.getRole() + " -> " + collection.getCollectionTable().getName());
/*      */     
/*      */ 
/*  654 */     bindFilters(false);
/*  655 */     bindCollectionSecondPass(collection, null, fkJoinColumns, cascadeDeleteEnabled, this.property, this.mappings);
/*  656 */     if ((!collection.isInverse()) && (!collection.getKey().isNullable()))
/*      */     {
/*      */ 
/*  659 */       String entityName = oneToMany.getReferencedEntityName();
/*  660 */       PersistentClass referenced = this.mappings.getClass(entityName);
/*  661 */       Backref prop = new Backref();
/*  662 */       prop.setName('_' + fkJoinColumns[0].getPropertyName() + "Backref");
/*  663 */       prop.setUpdateable(false);
/*  664 */       prop.setSelectable(false);
/*  665 */       prop.setCollectionRole(collection.getRole());
/*  666 */       prop.setEntityName(collection.getOwner().getEntityName());
/*  667 */       prop.setValue(collection.getKey());
/*  668 */       referenced.addProperty(prop);
/*      */     }
/*      */   }
/*      */   
/*      */   private void bindFilters(boolean hasAssociationTable)
/*      */   {
/*  674 */     Filter simpleFilter = (Filter)this.property.getAnnotation(Filter.class);
/*      */     
/*      */ 
/*      */ 
/*  678 */     if (simpleFilter != null) {
/*  679 */       if (hasAssociationTable) {
/*  680 */         this.collection.addManyToManyFilter(simpleFilter.name(), getCondition(simpleFilter));
/*      */       }
/*      */       else {
/*  683 */         this.collection.addFilter(simpleFilter.name(), getCondition(simpleFilter));
/*      */       }
/*      */     }
/*  686 */     Filters filters = (Filters)this.property.getAnnotation(Filters.class);
/*  687 */     if (filters != null) {
/*  688 */       for (Filter filter : filters.value()) {
/*  689 */         if (hasAssociationTable) {
/*  690 */           this.collection.addManyToManyFilter(filter.name(), getCondition(filter));
/*      */         }
/*      */         else {
/*  693 */           this.collection.addFilter(filter.name(), getCondition(filter));
/*      */         }
/*      */       }
/*      */     }
/*  697 */     FilterJoinTable simpleFilterJoinTable = (FilterJoinTable)this.property.getAnnotation(FilterJoinTable.class);
/*  698 */     if (simpleFilterJoinTable != null) {
/*  699 */       if (hasAssociationTable) {
/*  700 */         this.collection.addFilter(simpleFilterJoinTable.name(), getCondition(simpleFilterJoinTable));
/*      */       }
/*      */       else {
/*  703 */         throw new AnnotationException("Illegal use of @FilterJoinTable on an association without join table:" + StringHelper.qualify(this.propertyHolder.getPath(), this.propertyName));
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  709 */     FilterJoinTables filterJoinTables = (FilterJoinTables)this.property.getAnnotation(FilterJoinTables.class);
/*  710 */     if (filterJoinTables != null) {
/*  711 */       for (FilterJoinTable filter : filterJoinTables.value()) {
/*  712 */         if (hasAssociationTable) {
/*  713 */           this.collection.addFilter(filter.name(), getCondition(filter));
/*      */         }
/*      */         else {
/*  716 */           throw new AnnotationException("Illegal use of @FilterJoinTable on an association without join table:" + StringHelper.qualify(this.propertyHolder.getPath(), this.propertyName));
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  724 */     Where where = (Where)this.property.getAnnotation(Where.class);
/*  725 */     String whereClause = where == null ? null : where.clause();
/*  726 */     if (StringHelper.isNotEmpty(whereClause)) {
/*  727 */       if (hasAssociationTable) {
/*  728 */         this.collection.setManyToManyWhere(whereClause);
/*      */       }
/*      */       else {
/*  731 */         this.collection.setWhere(whereClause);
/*      */       }
/*      */     }
/*      */     
/*  735 */     WhereJoinTable whereJoinTable = (WhereJoinTable)this.property.getAnnotation(WhereJoinTable.class);
/*  736 */     String whereJoinTableClause = whereJoinTable == null ? null : whereJoinTable.clause();
/*  737 */     if (StringHelper.isNotEmpty(whereJoinTableClause)) {
/*  738 */       if (hasAssociationTable) {
/*  739 */         this.collection.setWhere(whereJoinTableClause);
/*      */       }
/*      */       else {
/*  742 */         throw new AnnotationException("Illegal use of @WhereJoinTable on an association without join table:" + StringHelper.qualify(this.propertyHolder.getPath(), this.propertyName));
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private String getCondition(FilterJoinTable filter)
/*      */   {
/*  761 */     String name = filter.name();
/*  762 */     String cond = filter.condition();
/*  763 */     return getCondition(cond, name);
/*      */   }
/*      */   
/*      */   private String getCondition(Filter filter)
/*      */   {
/*  768 */     String name = filter.name();
/*  769 */     String cond = filter.condition();
/*  770 */     return getCondition(cond, name);
/*      */   }
/*      */   
/*      */   private String getCondition(String cond, String name) {
/*  774 */     if (BinderHelper.isDefault(cond)) {
/*  775 */       cond = this.mappings.getFilterDefinition(name).getDefaultFilterCondition();
/*  776 */       if (StringHelper.isEmpty(cond)) {
/*  777 */         throw new AnnotationException("no filter condition found for filter " + name + " in " + StringHelper.qualify(this.propertyHolder.getPath(), this.propertyName));
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  783 */     return cond;
/*      */   }
/*      */   
/*      */   public void setCache(Cache cacheAnn) {
/*  787 */     if (cacheAnn != null) {
/*  788 */       this.cacheRegionName = (BinderHelper.isDefault(cacheAnn.region()) ? null : cacheAnn.region());
/*  789 */       this.cacheConcurrencyStrategy = EntityBinder.getCacheConcurrencyStrategy(cacheAnn.usage());
/*      */     }
/*      */     else {
/*  792 */       this.cacheConcurrencyStrategy = null;
/*  793 */       this.cacheRegionName = null;
/*      */     }
/*      */   }
/*      */   
/*      */   public void setOneToMany(boolean oneToMany) {
/*  798 */     this.oneToMany = oneToMany;
/*      */   }
/*      */   
/*      */   public void setIndexColumn(IndexColumn indexColumn) {
/*  802 */     this.indexColumn = indexColumn;
/*      */   }
/*      */   
/*      */   public void setMapKey(javax.persistence.MapKey key) {
/*  806 */     if (key != null) {
/*  807 */       this.mapKeyPropertyName = key.name();
/*      */     }
/*      */   }
/*      */   
/*      */   private static String buildOrderByClauseFromHql(String hqlOrderBy, PersistentClass associatedClass, String role) {
/*  812 */     String orderByString = null;
/*  813 */     if (hqlOrderBy != null) {
/*  814 */       List<String> properties = new ArrayList();
/*  815 */       List<String> ordering = new ArrayList();
/*  816 */       StringBuilder orderByBuffer = new StringBuilder();
/*  817 */       int index; if (hqlOrderBy.length() == 0)
/*      */       {
/*  819 */         Iterator it = associatedClass.getIdentifier().getColumnIterator();
/*  820 */         while (it.hasNext()) {
/*  821 */           Selectable col = (Selectable)it.next();
/*  822 */           orderByBuffer.append(col.getText()).append(" asc").append(", ");
/*      */         }
/*      */       }
/*      */       else {
/*  826 */         StringTokenizer st = new StringTokenizer(hqlOrderBy, " ,", false);
/*  827 */         String currentOrdering = null;
/*      */         
/*  829 */         while (st.hasMoreTokens()) {
/*  830 */           String token = st.nextToken();
/*  831 */           if (isNonPropertyToken(token)) {
/*  832 */             if (currentOrdering != null) {
/*  833 */               throw new AnnotationException("Error while parsing HQL orderBy clause: " + hqlOrderBy + " (" + role + ")");
/*      */             }
/*      */             
/*      */ 
/*      */ 
/*  838 */             currentOrdering = token;
/*      */           }
/*      */           else
/*      */           {
/*  842 */             if (currentOrdering == null)
/*      */             {
/*  844 */               ordering.add("asc");
/*      */             }
/*      */             else {
/*  847 */               ordering.add(currentOrdering);
/*  848 */               currentOrdering = null;
/*      */             }
/*  850 */             properties.add(token);
/*      */           }
/*      */         }
/*  853 */         ordering.remove(0);
/*      */         
/*  855 */         if (currentOrdering == null)
/*      */         {
/*  857 */           ordering.add("asc");
/*      */         }
/*      */         else {
/*  860 */           ordering.add(currentOrdering);
/*  861 */           currentOrdering = null;
/*      */         }
/*  863 */         index = 0;
/*      */         
/*  865 */         for (String property : properties) {
/*  866 */           Property p = BinderHelper.findPropertyByName(associatedClass, property);
/*  867 */           if (p == null) {
/*  868 */             throw new AnnotationException("property from @OrderBy clause not found: " + associatedClass.getEntityName() + "." + property);
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*  873 */           PersistentClass pc = p.getPersistentClass();
/*      */           String table;
/*  875 */           String table; if (pc == null)
/*      */           {
/*      */ 
/*      */ 
/*  879 */             table = "";
/*      */           } else { String table;
/*  881 */             if (pc != associatedClass) {
/*  882 */               table = pc.getTable().getQuotedName() + ".";
/*      */             }
/*      */             else
/*  885 */               table = "";
/*      */           }
/*  887 */           Iterator propertyColumns = p.getColumnIterator();
/*  888 */           while (propertyColumns.hasNext()) {
/*  889 */             Selectable column = (Selectable)propertyColumns.next();
/*  890 */             orderByBuffer.append(table).append(column.getText()).append(" ").append((String)ordering.get(index)).append(", ");
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*  896 */           index++;
/*      */         }
/*      */       }
/*  899 */       orderByString = orderByBuffer.substring(0, orderByBuffer.length() - 2);
/*      */     }
/*  901 */     return orderByString;
/*      */   }
/*      */   
/*      */   private static String buildOrderByClauseFromHql(String hqlOrderBy, Component component, String role) {
/*  905 */     String orderByString = null;
/*  906 */     if (hqlOrderBy != null) {
/*  907 */       List<String> properties = new ArrayList();
/*  908 */       List<String> ordering = new ArrayList();
/*  909 */       StringBuilder orderByBuffer = new StringBuilder();
/*  910 */       if (hqlOrderBy.length() != 0)
/*      */       {
/*      */ 
/*      */ 
/*  914 */         StringTokenizer st = new StringTokenizer(hqlOrderBy, " ,", false);
/*  915 */         String currentOrdering = null;
/*      */         
/*  917 */         while (st.hasMoreTokens()) {
/*  918 */           String token = st.nextToken();
/*  919 */           if (isNonPropertyToken(token)) {
/*  920 */             if (currentOrdering != null) {
/*  921 */               throw new AnnotationException("Error while parsing HQL orderBy clause: " + hqlOrderBy + " (" + role + ")");
/*      */             }
/*      */             
/*      */ 
/*      */ 
/*  926 */             currentOrdering = token;
/*      */           }
/*      */           else
/*      */           {
/*  930 */             if (currentOrdering == null)
/*      */             {
/*  932 */               ordering.add("asc");
/*      */             }
/*      */             else {
/*  935 */               ordering.add(currentOrdering);
/*  936 */               currentOrdering = null;
/*      */             }
/*  938 */             properties.add(token);
/*      */           }
/*      */         }
/*  941 */         ordering.remove(0);
/*      */         
/*  943 */         if (currentOrdering == null)
/*      */         {
/*  945 */           ordering.add("asc");
/*      */         }
/*      */         else {
/*  948 */           ordering.add(currentOrdering);
/*  949 */           currentOrdering = null;
/*      */         }
/*  951 */         int index = 0;
/*      */         
/*  953 */         for (String property : properties) {
/*  954 */           Property p = component.getProperty(property);
/*  955 */           if (p == null) {
/*  956 */             throw new AnnotationException("property from @OrderBy clause not found: " + role + "." + property);
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*  962 */           Iterator propertyColumns = p.getColumnIterator();
/*  963 */           while (propertyColumns.hasNext()) {
/*  964 */             Selectable column = (Selectable)propertyColumns.next();
/*  965 */             orderByBuffer.append(column.getText()).append(" ").append((String)ordering.get(index)).append(", ");
/*      */           }
/*      */           
/*      */ 
/*      */ 
/*  970 */           index++;
/*      */         }
/*      */         
/*  973 */         if (orderByBuffer.length() >= 2) {
/*  974 */           orderByString = orderByBuffer.substring(0, orderByBuffer.length() - 2);
/*      */         }
/*      */       }
/*      */     }
/*  978 */     return orderByString;
/*      */   }
/*      */   
/*      */   private static boolean isNonPropertyToken(String token) {
/*  982 */     if (" ".equals(token)) return true;
/*  983 */     if (",".equals(token)) return true;
/*  984 */     if (token.equalsIgnoreCase("desc")) return true;
/*  985 */     if (token.equalsIgnoreCase("asc")) return true;
/*  986 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static SimpleValue buildCollectionKey(org.hibernate.mapping.Collection collValue, Ejb3JoinColumn[] joinColumns, boolean cascadeDeleteEnabled, XProperty property, ExtendedMappings mappings)
/*      */   {
/*  997 */     if ((joinColumns.length > 0) && (StringHelper.isNotEmpty(joinColumns[0].getMappedBy()))) {
/*  998 */       String entityName = joinColumns[0].getManyToManyOwnerSideEntityName() != null ? "inverse__" + joinColumns[0].getManyToManyOwnerSideEntityName() : joinColumns[0].getPropertyHolder().getEntityName();
/*      */       
/*      */ 
/* 1001 */       String propRef = mappings.getPropertyReferencedAssociation(entityName, joinColumns[0].getMappedBy());
/*      */       
/*      */ 
/*      */ 
/* 1005 */       if (propRef != null) {
/* 1006 */         collValue.setReferencedPropertyName(propRef);
/* 1007 */         mappings.addPropertyReference(collValue.getOwnerEntityName(), propRef);
/*      */       }
/*      */     }
/* 1010 */     String propRef = collValue.getReferencedPropertyName();
/* 1011 */     KeyValue keyVal; KeyValue keyVal; if (propRef == null) {
/* 1012 */       keyVal = collValue.getOwner().getIdentifier();
/*      */     }
/*      */     else {
/* 1015 */       keyVal = (KeyValue)collValue.getOwner().getRecursiveProperty(propRef).getValue();
/*      */     }
/*      */     
/*      */ 
/* 1019 */     DependantValue key = new DependantValue(collValue.getCollectionTable(), keyVal);
/* 1020 */     key.setTypeName(null);
/* 1021 */     Ejb3Column.checkPropertyConsistency(joinColumns, collValue.getOwnerEntityName());
/* 1022 */     key.setNullable((joinColumns.length == 0) || (joinColumns[0].isNullable()));
/* 1023 */     key.setUpdateable((joinColumns.length == 0) || (joinColumns[0].isUpdatable()));
/* 1024 */     key.setCascadeDeleteEnabled(cascadeDeleteEnabled);
/* 1025 */     collValue.setKey(key);
/* 1026 */     ForeignKey fk = property != null ? (ForeignKey)property.getAnnotation(ForeignKey.class) : null;
/* 1027 */     String fkName = fk != null ? fk.name() : "";
/* 1028 */     if (!BinderHelper.isDefault(fkName)) key.setForeignKeyName(fkName);
/* 1029 */     return key;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void bindManyToManySecondPass(org.hibernate.mapping.Collection collValue, Map persistentClasses, Ejb3JoinColumn[] joinColumns, Ejb3JoinColumn[] inverseJoinColumns, Ejb3Column[] elementColumns, boolean isEmbedded, XClass collType, boolean ignoreNotFound, boolean unique, boolean cascadeDeleteEnabled, TableBinder associationTableBinder, XProperty property, PropertyHolder parentPropertyHolder, String hqlOrderBy, ExtendedMappings mappings)
/*      */     throws MappingException
/*      */   {
/* 1046 */     PersistentClass collectionEntity = (PersistentClass)persistentClasses.get(collType.getName());
/* 1047 */     boolean isCollectionOfEntities = collectionEntity != null;
/* 1048 */     ManyToAny anyAnn = (ManyToAny)property.getAnnotation(ManyToAny.class);
/* 1049 */     if (log.isDebugEnabled()) {
/* 1050 */       String path = collValue.getOwnerEntityName() + "." + joinColumns[0].getPropertyName();
/* 1051 */       if ((isCollectionOfEntities) && (unique)) {
/* 1052 */         log.debug("Binding a OneToMany: " + path + " through an association table");
/*      */       }
/* 1054 */       else if (isCollectionOfEntities) {
/* 1055 */         log.debug("Binding as ManyToMany: " + path);
/*      */       }
/* 1057 */       else if (anyAnn != null) {
/* 1058 */         log.debug("Binding a ManyToAny: " + path);
/*      */       }
/*      */       else {
/* 1061 */         log.debug("Binding a collection of element: " + path);
/*      */       }
/*      */     }
/*      */     
/* 1065 */     if (!isCollectionOfEntities) {
/* 1066 */       if ((property.isAnnotationPresent(ManyToMany.class)) || (property.isAnnotationPresent(javax.persistence.OneToMany.class))) {
/* 1067 */         String path = collValue.getOwnerEntityName() + "." + joinColumns[0].getPropertyName();
/* 1068 */         throw new AnnotationException("Use of @OneToMany or @ManyToMany targeting an unmapped class: " + path + "[" + collType + "]");
/*      */       }
/*      */       
/*      */ 
/* 1072 */       if (anyAnn != null) {
/* 1073 */         if (!property.isAnnotationPresent(JoinTable.class)) {
/* 1074 */           String path = collValue.getOwnerEntityName() + "." + joinColumns[0].getPropertyName();
/* 1075 */           throw new AnnotationException("@JoinTable is mandatory when @ManyToAny is used: " + path);
/*      */         }
/*      */         
/*      */       }
/*      */       else
/*      */       {
/* 1081 */         JoinTable joinTableAnn = (JoinTable)property.getAnnotation(JoinTable.class);
/* 1082 */         if ((joinTableAnn != null) && (joinTableAnn.inverseJoinColumns().length > 0)) {
/* 1083 */           String path = collValue.getOwnerEntityName() + "." + joinColumns[0].getPropertyName();
/* 1084 */           throw new AnnotationException("Use of @JoinTable.inverseJoinColumns targeting an unmapped class: " + path + "[" + collType + "]");
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1091 */     boolean mappedBy = !BinderHelper.isDefault(joinColumns[0].getMappedBy());
/* 1092 */     if (mappedBy) {
/* 1093 */       if (!isCollectionOfEntities) {
/* 1094 */         StringBuilder error = new StringBuilder(80).append("Collection of elements must not have mappedBy or association reference an unmapped entity: ").append(collValue.getOwnerEntityName()).append(".").append(joinColumns[0].getPropertyName());
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1101 */         throw new AnnotationException(error.toString());
/*      */       }
/*      */       Property otherSideProperty;
/*      */       try {
/* 1105 */         otherSideProperty = collectionEntity.getRecursiveProperty(joinColumns[0].getMappedBy());
/*      */       }
/*      */       catch (MappingException e) {
/* 1108 */         StringBuilder error = new StringBuilder(80);
/* 1109 */         error.append("mappedBy reference an unknown target entity property: ").append(collType).append(".").append(joinColumns[0].getMappedBy()).append(" in ").append(collValue.getOwnerEntityName()).append(".").append(joinColumns[0].getPropertyName());
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1115 */         throw new AnnotationException(error.toString()); }
/*      */       Table table;
/*      */       Table table;
/* 1118 */       if ((otherSideProperty.getValue() instanceof org.hibernate.mapping.Collection))
/*      */       {
/* 1120 */         table = ((org.hibernate.mapping.Collection)otherSideProperty.getValue()).getCollectionTable();
/*      */       }
/*      */       else
/*      */       {
/* 1124 */         table = otherSideProperty.getValue().getTable();
/*      */       }
/* 1126 */       collValue.setCollectionTable(table);
/* 1127 */       String entityName = collectionEntity.getEntityName();
/* 1128 */       for (Ejb3JoinColumn column : joinColumns)
/*      */       {
/* 1130 */         column.setManyToManyOwnerSideEntityName(entityName);
/*      */       }
/*      */       
/*      */     }
/*      */     else
/*      */     {
/* 1136 */       for (Ejb3JoinColumn column : joinColumns) {
/* 1137 */         String mappedByProperty = mappings.getFromMappedBy(collValue.getOwnerEntityName(), column.getPropertyName());
/*      */         
/*      */ 
/* 1140 */         Table ownerTable = collValue.getOwner().getTable();
/* 1141 */         column.setMappedBy(collValue.getOwner().getEntityName(), mappings.getLogicalTableName(ownerTable), mappedByProperty);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1148 */       if (StringHelper.isEmpty(associationTableBinder.getName()))
/*      */       {
/* 1150 */         associationTableBinder.setDefaultName(collValue.getOwner().getEntityName(), mappings.getLogicalTableName(collValue.getOwner().getTable()), collectionEntity != null ? collectionEntity.getEntityName() : null, collectionEntity != null ? mappings.getLogicalTableName(collectionEntity.getTable()) : null, joinColumns[0].getPropertyName());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1158 */       collValue.setCollectionTable(associationTableBinder.bind());
/*      */     }
/* 1160 */     bindFilters(isCollectionOfEntities);
/* 1161 */     bindCollectionSecondPass(collValue, collectionEntity, joinColumns, cascadeDeleteEnabled, property, mappings);
/*      */     
/* 1163 */     ManyToOne element = null;
/* 1164 */     if (isCollectionOfEntities) {
/* 1165 */       element = new ManyToOne(collValue.getCollectionTable());
/*      */       
/* 1167 */       collValue.setElement(element);
/* 1168 */       element.setReferencedEntityName(collType.getName());
/*      */       
/*      */ 
/*      */ 
/* 1172 */       element.setFetchMode(org.hibernate.FetchMode.JOIN);
/* 1173 */       element.setLazy(false);
/* 1174 */       element.setIgnoreNotFound(ignoreNotFound);
/* 1175 */       if (StringHelper.isNotEmpty(hqlOrderBy)) {
/* 1176 */         collValue.setManyToManyOrdering(buildOrderByClauseFromHql(hqlOrderBy, collectionEntity, collValue.getRole()));
/*      */       }
/*      */       
/*      */ 
/* 1180 */       ForeignKey fk = property != null ? (ForeignKey)property.getAnnotation(ForeignKey.class) : null;
/* 1181 */       String fkName = fk != null ? fk.inverseName() : "";
/* 1182 */       if (!BinderHelper.isDefault(fkName)) element.setForeignKeyName(fkName);
/*      */     }
/* 1184 */     else if (anyAnn != null)
/*      */     {
/*      */ 
/* 1187 */       PropertyData inferredData = new PropertyInferredData(property, "unsupported", mappings.getReflectionManager());
/*      */       
/* 1189 */       for (Ejb3Column column : inverseJoinColumns) {
/* 1190 */         column.setTable(collValue.getCollectionTable());
/*      */       }
/* 1192 */       Any any = BinderHelper.buildAnyValue(anyAnn.metaDef(), inverseJoinColumns, anyAnn.metaColumn(), inferredData, cascadeDeleteEnabled, Nullability.NO_CONSTRAINT, this.propertyHolder, new EntityBinder(), true, mappings);
/*      */       
/*      */ 
/* 1195 */       collValue.setElement(any);
/*      */ 
/*      */ 
/*      */ 
/*      */     }
/*      */     else
/*      */     {
/*      */ 
/*      */ 
/* 1204 */       PropertyHolder holder = null;
/* 1205 */       XClass elementClass; XClass elementClass; AnnotatedClassType classType; if (BinderHelper.PRIMITIVE_NAMES.contains(collType.getName())) {
/* 1206 */         AnnotatedClassType classType = AnnotatedClassType.NONE;
/* 1207 */         elementClass = null;
/*      */       }
/*      */       else {
/* 1210 */         elementClass = collType;
/* 1211 */         classType = mappings.getClassType(elementClass);
/*      */         
/* 1213 */         holder = PropertyHolderBuilder.buildPropertyHolder(collValue, collValue.getRole(), elementClass, property, parentPropertyHolder, mappings);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1220 */         boolean attributeOverride = (property.isAnnotationPresent(AttributeOverride.class)) || (property.isAnnotationPresent(AttributeOverrides.class));
/*      */         
/* 1222 */         if ((isEmbedded) || (attributeOverride)) {
/* 1223 */           classType = AnnotatedClassType.EMBEDDABLE;
/*      */         }
/*      */       }
/*      */       
/* 1227 */       if (AnnotatedClassType.EMBEDDABLE.equals(classType)) {
/* 1228 */         EntityBinder entityBinder = new EntityBinder();
/* 1229 */         PersistentClass owner = collValue.getOwner();
/*      */         
/*      */         boolean isPropertyAnnotated;
/*      */         
/* 1233 */         if (owner.getIdentifierProperty() != null) {
/* 1234 */           isPropertyAnnotated = owner.getIdentifierProperty().getPropertyAccessorName().equals("property");
/*      */         } else { boolean isPropertyAnnotated;
/* 1236 */           if ((owner.getIdentifierMapper() != null) && (owner.getIdentifierMapper().getPropertySpan() > 0)) {
/* 1237 */             Property prop = (Property)owner.getIdentifierMapper().getPropertyIterator().next();
/* 1238 */             isPropertyAnnotated = prop.getPropertyAccessorName().equals("property");
/*      */           }
/*      */           else {
/* 1241 */             throw new AssertionFailure("Unable to guess collection property accessor name");
/*      */           }
/*      */         }
/*      */         boolean isPropertyAnnotated;
/* 1245 */         PropertyData inferredData = new PropertyPreloadedData("property", "element", elementClass);
/*      */         
/* 1247 */         Component component = AnnotationBinder.fillComponent(holder, inferredData, isPropertyAnnotated, isPropertyAnnotated ? "property" : "field", true, entityBinder, false, false, true, mappings);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1253 */         collValue.setElement(component);
/*      */         
/* 1255 */         if (StringHelper.isNotEmpty(hqlOrderBy)) {
/* 1256 */           String path = collValue.getOwnerEntityName() + "." + joinColumns[0].getPropertyName();
/* 1257 */           String orderBy = buildOrderByClauseFromHql(hqlOrderBy, component, path);
/* 1258 */           if (orderBy != null) {
/* 1259 */             collValue.setOrderBy(orderBy);
/*      */           }
/*      */         }
/*      */       }
/*      */       else {
/* 1264 */         SimpleValueBinder elementBinder = new SimpleValueBinder();
/* 1265 */         elementBinder.setMappings(mappings);
/* 1266 */         elementBinder.setReturnedClassName(collType.getName());
/* 1267 */         if ((elementColumns == null) || (elementColumns.length == 0)) {
/* 1268 */           elementColumns = new Ejb3Column[1];
/* 1269 */           Ejb3Column column = new Ejb3Column();
/* 1270 */           column.setImplicit(false);
/*      */           
/* 1272 */           column.setNullable(true);
/* 1273 */           column.setLength(255);
/* 1274 */           column.setLogicalColumnName("elt");
/*      */           
/* 1276 */           column.setJoins(new HashMap());
/* 1277 */           column.setMappings(mappings);
/* 1278 */           column.bind();
/* 1279 */           elementColumns[0] = column;
/*      */         }
/*      */         
/* 1282 */         for (Ejb3Column column : elementColumns) {
/* 1283 */           column.setTable(collValue.getCollectionTable());
/*      */         }
/* 1285 */         elementBinder.setColumns(elementColumns);
/* 1286 */         elementBinder.setType(property, elementClass);
/* 1287 */         collValue.setElement(elementBinder.make());
/*      */       }
/*      */     }
/*      */     
/* 1291 */     checkFilterConditions(collValue);
/*      */     
/*      */ 
/* 1294 */     if (isCollectionOfEntities) {
/* 1295 */       bindManytoManyInverseFk(collectionEntity, inverseJoinColumns, element, unique, mappings);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private static void checkFilterConditions(org.hibernate.mapping.Collection collValue)
/*      */   {
/* 1302 */     if (((collValue.getFilterMap().size() != 0) || (StringHelper.isNotEmpty(collValue.getWhere()))) && (collValue.getFetchMode() == org.hibernate.FetchMode.JOIN) && (!(collValue.getElement() instanceof SimpleValue)) && (collValue.getElement().getFetchMode() != org.hibernate.FetchMode.JOIN))
/*      */     {
/*      */ 
/*      */ 
/* 1306 */       throw new MappingException("@ManyToMany or @CollectionOfElements defining filter or where without join fetching not valid within collection using join fetching[" + collValue.getRole() + "]");
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static void bindCollectionSecondPass(org.hibernate.mapping.Collection collValue, PersistentClass collectionEntity, Ejb3JoinColumn[] joinColumns, boolean cascadeDeleteEnabled, XProperty property, ExtendedMappings mappings)
/*      */   {
/* 1318 */     BinderHelper.createSyntheticPropertyReference(joinColumns, collValue.getOwner(), collectionEntity, collValue, false, mappings);
/*      */     
/*      */ 
/* 1321 */     SimpleValue key = buildCollectionKey(collValue, joinColumns, cascadeDeleteEnabled, property, mappings);
/* 1322 */     TableBinder.bindFk(collValue.getOwner(), collectionEntity, joinColumns, key, false, mappings);
/*      */   }
/*      */   
/*      */   public void setCascadeDeleteEnabled(boolean onDeleteCascade) {
/* 1326 */     this.cascadeDeleteEnabled = onDeleteCascade;
/*      */   }
/*      */   
/*      */   private String safeCollectionRole() {
/* 1330 */     if (this.propertyHolder != null) {
/* 1331 */       return this.propertyHolder.getEntityName() + "." + this.propertyName;
/*      */     }
/*      */     
/* 1334 */     return "";
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static void bindManytoManyInverseFk(PersistentClass referencedEntity, Ejb3JoinColumn[] columns, SimpleValue value, boolean unique, ExtendedMappings mappings)
/*      */   {
/* 1349 */     String mappedBy = columns[0].getMappedBy();
/* 1350 */     if (StringHelper.isNotEmpty(mappedBy)) {
/* 1351 */       Property property = referencedEntity.getRecursiveProperty(mappedBy);
/*      */       Iterator mappedByColumns;
/* 1353 */       Iterator mappedByColumns; if ((property.getValue() instanceof org.hibernate.mapping.Collection)) {
/* 1354 */         mappedByColumns = ((org.hibernate.mapping.Collection)property.getValue()).getKey().getColumnIterator();
/*      */       }
/*      */       else
/*      */       {
/* 1358 */         Iterator joinsIt = referencedEntity.getJoinIterator();
/* 1359 */         KeyValue key = null;
/* 1360 */         while (joinsIt.hasNext()) {
/* 1361 */           Join join = (Join)joinsIt.next();
/* 1362 */           if (join.containsProperty(property)) {
/* 1363 */             key = join.getKey();
/* 1364 */             break;
/*      */           }
/*      */         }
/* 1367 */         if (key == null) key = property.getPersistentClass().getIdentifier();
/* 1368 */         mappedByColumns = key.getColumnIterator();
/*      */       }
/* 1370 */       while (mappedByColumns.hasNext()) {
/* 1371 */         Column column = (Column)mappedByColumns.next();
/* 1372 */         columns[0].linkValueUsingAColumnCopy(column, value);
/*      */       }
/* 1374 */       String referencedPropertyName = mappings.getPropertyReferencedAssociation("inverse__" + referencedEntity.getEntityName(), mappedBy);
/*      */       
/*      */ 
/*      */ 
/* 1378 */       if (referencedPropertyName != null)
/*      */       {
/* 1380 */         ((ManyToOne)value).setReferencedPropertyName(referencedPropertyName);
/* 1381 */         mappings.addUniquePropertyReference(referencedEntity.getEntityName(), referencedPropertyName);
/*      */       }
/* 1383 */       value.createForeignKey();
/*      */     }
/*      */     else {
/* 1386 */       BinderHelper.createSyntheticPropertyReference(columns, referencedEntity, null, value, true, mappings);
/* 1387 */       TableBinder.bindFk(referencedEntity, null, columns, value, unique, mappings);
/*      */     }
/*      */   }
/*      */   
/*      */   public void setFkJoinColumns(Ejb3JoinColumn[] ejb3JoinColumns) {
/* 1392 */     this.fkJoinColumns = ejb3JoinColumns;
/*      */   }
/*      */   
/*      */   public void setExplicitAssociationTable(boolean explicitAssocTable) {
/* 1396 */     this.isExplicitAssociationTable = explicitAssocTable;
/*      */   }
/*      */   
/*      */   public void setElementColumns(Ejb3Column[] elementColumns) {
/* 1400 */     this.elementColumns = elementColumns;
/*      */   }
/*      */   
/*      */   public void setEmbedded(boolean annotationPresent) {
/* 1404 */     this.isEmbedded = annotationPresent;
/*      */   }
/*      */   
/*      */   public void setProperty(XProperty property) {
/* 1408 */     this.property = property;
/*      */   }
/*      */   
/*      */   public void setIgnoreNotFound(boolean ignoreNotFound) {
/* 1412 */     this.ignoreNotFound = ignoreNotFound;
/*      */   }
/*      */   
/*      */   public void setMapKeyColumns(Ejb3Column[] mapKeyColumns) {
/* 1416 */     this.mapKeyColumns = mapKeyColumns;
/*      */   }
/*      */   
/*      */   public void setMapKeyManyToManyColumns(Ejb3JoinColumn[] mapJoinColumns) {
/* 1420 */     this.mapKeyManyToManyColumns = mapJoinColumns;
/*      */   }
/*      */   
/*      */   public void setLocalGenerators(HashMap<String, IdGenerator> localGenerators) {
/* 1424 */     this.localGenerators = localGenerators;
/*      */   }
/*      */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\annotations\CollectionBinder.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */