/*     */ package org.hibernate.cfg.annotations;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import javax.persistence.UniqueConstraint;
/*     */ import org.apache.commons.logging.Log;
/*     */ import org.apache.commons.logging.LogFactory;
/*     */ import org.hibernate.AnnotationException;
/*     */ import org.hibernate.AssertionFailure;
/*     */ import org.hibernate.annotations.Index;
/*     */ import org.hibernate.annotations.common.util.StringHelper;
/*     */ import org.hibernate.cfg.BinderHelper;
/*     */ import org.hibernate.cfg.Ejb3JoinColumn;
/*     */ import org.hibernate.cfg.ExtendedMappings;
/*     */ import org.hibernate.cfg.IndexOrUniqueKeySecondPass;
/*     */ import org.hibernate.cfg.NamingStrategy;
/*     */ import org.hibernate.cfg.PropertyHolder;
/*     */ import org.hibernate.mapping.Collection;
/*     */ import org.hibernate.mapping.Column;
/*     */ import org.hibernate.mapping.DependantValue;
/*     */ import org.hibernate.mapping.JoinedSubclass;
/*     */ import org.hibernate.mapping.KeyValue;
/*     */ import org.hibernate.mapping.PersistentClass;
/*     */ import org.hibernate.mapping.Property;
/*     */ import org.hibernate.mapping.SimpleValue;
/*     */ import org.hibernate.mapping.Table;
/*     */ import org.hibernate.mapping.ToOne;
/*     */ import org.hibernate.mapping.Value;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class TableBinder
/*     */ {
/*  37 */   private static Log log = LogFactory.getLog(TableBinder.class);
/*     */   private String schema;
/*     */   private String catalog;
/*     */   private String name;
/*     */   private boolean isAbstract;
/*     */   private List<String[]> uniqueConstraints;
/*     */   String constraints;
/*     */   Table denormalizedSuperTable;
/*     */   ExtendedMappings mappings;
/*     */   private String ownerEntityTable;
/*     */   private String associatedEntityTable;
/*     */   private String propertyName;
/*     */   private String ownerEntity;
/*     */   private String associatedEntity;
/*     */   
/*     */   public void setSchema(String schema) {
/*  53 */     this.schema = schema;
/*     */   }
/*     */   
/*     */   public void setCatalog(String catalog) {
/*  57 */     this.catalog = catalog;
/*     */   }
/*     */   
/*     */   public String getName() {
/*  61 */     return this.name;
/*     */   }
/*     */   
/*     */   public void setName(String name) {
/*  65 */     this.name = name;
/*     */   }
/*     */   
/*     */   public void setAbstract(boolean anAbstract) {
/*  69 */     this.isAbstract = anAbstract;
/*     */   }
/*     */   
/*     */   public void setUniqueConstraints(UniqueConstraint[] uniqueConstraints) {
/*  73 */     this.uniqueConstraints = buildUniqueConstraints(uniqueConstraints);
/*     */   }
/*     */   
/*     */   public void setConstraints(String constraints) {
/*  77 */     this.constraints = constraints;
/*     */   }
/*     */   
/*     */   public void setDenormalizedSuperTable(Table denormalizedSuperTable) {
/*  81 */     this.denormalizedSuperTable = denormalizedSuperTable;
/*     */   }
/*     */   
/*     */   public void setMappings(ExtendedMappings mappings) {
/*  85 */     this.mappings = mappings;
/*     */   }
/*     */   
/*     */ 
/*     */   public Table bind()
/*     */   {
/*  91 */     String unquotedOwnerTable = StringHelper.unquote(this.ownerEntityTable);
/*  92 */     String unquotedAssocTable = StringHelper.unquote(this.associatedEntityTable);
/*     */     
/*  94 */     String logicalName = this.mappings.getNamingStrategy().logicalCollectionTableName(this.name, unquotedOwnerTable, unquotedAssocTable, this.propertyName);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 100 */     if ((StringHelper.isQuoted(this.ownerEntityTable)) || (StringHelper.isQuoted(this.associatedEntityTable)))
/* 101 */       logicalName = StringHelper.quote(logicalName);
/*     */     String extendedName;
/*     */     String extendedName;
/* 104 */     if (this.name != null) {
/* 105 */       extendedName = this.mappings.getNamingStrategy().tableName(this.name);
/*     */     }
/*     */     else {
/* 108 */       extendedName = this.mappings.getNamingStrategy().collectionTableName(this.ownerEntity, unquotedOwnerTable, this.associatedEntity, unquotedAssocTable, this.propertyName);
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 116 */       if ((StringHelper.isQuoted(this.ownerEntityTable)) || (StringHelper.isQuoted(this.associatedEntityTable))) {
/* 117 */         extendedName = StringHelper.quote(extendedName);
/*     */       }
/*     */     }
/* 120 */     return fillTable(this.schema, this.catalog, extendedName, logicalName, this.isAbstract, this.uniqueConstraints, this.constraints, this.denormalizedSuperTable, this.mappings);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static Table fillTable(String schema, String catalog, String realTableName, String logicalName, boolean isAbstract, List uniqueConstraints, String constraints, Table denormalizedSuperTable, ExtendedMappings mappings)
/*     */   {
/* 131 */     schema = BinderHelper.isDefault(schema) ? mappings.getSchemaName() : schema;
/* 132 */     catalog = BinderHelper.isDefault(catalog) ? mappings.getCatalogName() : catalog;
/*     */     Table table;
/* 134 */     Table table; if (denormalizedSuperTable != null) {
/* 135 */       table = mappings.addDenormalizedTable(schema, catalog, realTableName, isAbstract, null, denormalizedSuperTable);
/*     */ 
/*     */ 
/*     */ 
/*     */     }
/*     */     else
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/* 145 */       table = mappings.addTable(schema, catalog, realTableName, null, isAbstract);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 153 */     if ((uniqueConstraints != null) && (uniqueConstraints.size() > 0)) {
/* 154 */       mappings.addUniqueConstraints(table, uniqueConstraints);
/*     */     }
/* 156 */     if (constraints != null) { table.addCheckConstraint(constraints);
/*     */     }
/* 158 */     if (logicalName != null) {
/* 159 */       mappings.addTableBinding(schema, catalog, logicalName, realTableName, denormalizedSuperTable);
/*     */     }
/* 161 */     return table;
/*     */   }
/*     */   
/*     */ 
/*     */   public static void bindFk(PersistentClass referencedEntity, PersistentClass destinationEntity, Ejb3JoinColumn[] columns, SimpleValue value, boolean unique, ExtendedMappings mappings)
/*     */   {
/*     */     PersistentClass associatedClass;
/*     */     
/*     */     PersistentClass associatedClass;
/* 170 */     if (destinationEntity != null)
/*     */     {
/* 172 */       associatedClass = destinationEntity;
/*     */     }
/*     */     else {
/* 175 */       associatedClass = columns[0].getPropertyHolder() == null ? null : columns[0].getPropertyHolder().getPersistentClass();
/*     */     }
/*     */     
/* 178 */     String mappedByProperty = columns[0].getMappedBy();
/* 179 */     if (StringHelper.isNotEmpty(mappedByProperty))
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/* 184 */       if (log.isDebugEnabled()) {
/* 185 */         log.debug("Retrieving property " + associatedClass.getEntityName() + "." + mappedByProperty);
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 190 */       Property property = associatedClass.getRecursiveProperty(columns[0].getMappedBy());
/*     */       Iterator mappedByColumns;
/* 192 */       Iterator mappedByColumns; if ((property.getValue() instanceof Collection)) {
/* 193 */         Collection collection = (Collection)property.getValue();
/* 194 */         Value element = collection.getElement();
/* 195 */         if (element == null) {
/* 196 */           throw new AnnotationException("Illegal use of mappedBy on both sides of the relationship: " + associatedClass.getEntityName() + "." + mappedByProperty);
/*     */         }
/*     */         
/*     */ 
/*     */ 
/* 201 */         mappedByColumns = element.getColumnIterator();
/*     */       }
/*     */       else {
/* 204 */         mappedByColumns = property.getValue().getColumnIterator();
/*     */       }
/* 206 */       while (mappedByColumns.hasNext()) {
/* 207 */         Column column = (Column)mappedByColumns.next();
/* 208 */         columns[0].overrideSqlTypeIfNecessary(column);
/* 209 */         columns[0].linkValueUsingAColumnCopy(column, value);
/*     */       }
/*     */     }
/* 212 */     else if (columns[0].isImplicit())
/*     */     {
/*     */       Iterator idColumns;
/*     */       
/*     */       Iterator idColumns;
/*     */       
/* 218 */       if ((referencedEntity instanceof JoinedSubclass)) {
/* 219 */         idColumns = ((JoinedSubclass)referencedEntity).getKey().getColumnIterator();
/*     */       }
/*     */       else {
/* 222 */         idColumns = referencedEntity.getIdentifier().getColumnIterator();
/*     */       }
/* 224 */       while (idColumns.hasNext()) {
/* 225 */         Column column = (Column)idColumns.next();
/* 226 */         columns[0].overrideSqlTypeIfNecessary(column);
/* 227 */         columns[0].linkValueUsingDefaultColumnNaming(column, referencedEntity, value);
/*     */       }
/*     */     }
/*     */     else {
/* 231 */       int fkEnum = Ejb3JoinColumn.checkReferencedColumnsType(columns, referencedEntity, mappings);
/*     */       
/* 233 */       if (2 == fkEnum) {
/*     */         String referencedPropertyName;
/* 235 */         if ((value instanceof ToOne)) {
/* 236 */           referencedPropertyName = ((ToOne)value).getReferencedPropertyName();
/*     */         } else { String referencedPropertyName;
/* 238 */           if ((value instanceof DependantValue)) {
/* 239 */             String propertyName = columns[0].getPropertyName();
/* 240 */             String referencedPropertyName; if (propertyName != null) {
/* 241 */               Collection collection = (Collection)referencedEntity.getRecursiveProperty(propertyName).getValue();
/*     */               
/* 243 */               referencedPropertyName = collection.getReferencedPropertyName();
/*     */             }
/*     */             else {
/* 246 */               throw new AnnotationException("SecondaryTable JoinColumn cannot reference a non primary key");
/*     */             }
/*     */           }
/*     */           else
/*     */           {
/* 251 */             throw new AssertionFailure("Do a property ref on an unexpected Value type: " + value.getClass().getName());
/*     */           }
/*     */         }
/*     */         
/*     */         String referencedPropertyName;
/* 256 */         if (referencedPropertyName == null) {
/* 257 */           throw new AssertionFailure("No property ref found while expected");
/*     */         }
/*     */         
/*     */ 
/* 261 */         Property synthProp = referencedEntity.getRecursiveProperty(referencedPropertyName);
/* 262 */         if (synthProp == null) {
/* 263 */           throw new AssertionFailure("Cannot find synthProp: " + referencedEntity.getEntityName() + "." + referencedPropertyName);
/*     */         }
/*     */         
/*     */ 
/* 267 */         linkJoinColumnWithValueOverridingNameIfImplicit(referencedEntity, synthProp.getColumnIterator(), columns, value);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       }
/* 273 */       else if (0 == fkEnum)
/*     */       {
/* 275 */         if (columns.length != referencedEntity.getIdentifier().getColumnSpan()) {
/* 276 */           throw new AnnotationException("A Foreign key refering " + referencedEntity.getEntityName() + " from " + associatedClass.getEntityName() + " has the wrong number of column. should be " + referencedEntity.getIdentifier().getColumnSpan());
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 283 */         linkJoinColumnWithValueOverridingNameIfImplicit(referencedEntity, referencedEntity.getIdentifier().getColumnIterator(), columns, value);
/*     */ 
/*     */ 
/*     */ 
/*     */       }
/*     */       else
/*     */       {
/*     */ 
/*     */ 
/* 292 */         Iterator idColItr = referencedEntity.getKey().getColumnIterator();
/*     */         
/* 294 */         Table table = referencedEntity.getTable();
/* 295 */         if (!idColItr.hasNext()) log.debug("No column in the identifier!");
/* 296 */         while (idColItr.hasNext()) {
/* 297 */           boolean match = false;
/*     */           
/* 299 */           Column col = (Column)idColItr.next();
/* 300 */           for (Ejb3JoinColumn joinCol : columns) {
/* 301 */             String referencedColumn = joinCol.getReferencedColumn();
/* 302 */             referencedColumn = mappings.getPhysicalColumnName(referencedColumn, table);
/* 303 */             if (referencedColumn.equals(col.getName()))
/*     */             {
/* 305 */               if (joinCol.isNameDeferred()) {
/* 306 */                 joinCol.linkValueUsingDefaultColumnNaming(col, referencedEntity, value);
/*     */ 
/*     */               }
/*     */               else
/*     */               {
/* 311 */                 joinCol.linkWithValue(value);
/*     */               }
/* 313 */               joinCol.overrideSqlTypeIfNecessary(col);
/* 314 */               match = true;
/* 315 */               break;
/*     */             }
/*     */           }
/* 318 */           if (!match) {
/* 319 */             throw new AnnotationException("Column name " + col.getName() + " of " + referencedEntity.getEntityName() + " not found in JoinColumns.referencedColumnName");
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 328 */     value.createForeignKey();
/* 329 */     if (unique == true) {
/* 330 */       createUniqueConstraint(value);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   private static void linkJoinColumnWithValueOverridingNameIfImplicit(PersistentClass referencedEntity, Iterator columnIterator, Ejb3JoinColumn[] columns, SimpleValue value)
/*     */   {
/* 337 */     for (Ejb3JoinColumn joinCol : columns) {
/* 338 */       Column synthCol = (Column)columnIterator.next();
/* 339 */       if (joinCol.isNameDeferred())
/*     */       {
/* 341 */         joinCol.linkValueUsingDefaultColumnNaming(synthCol, referencedEntity, value);
/*     */       }
/*     */       else {
/* 344 */         joinCol.linkWithValue(value);
/*     */       }
/* 346 */       joinCol.overrideSqlTypeIfNecessary(synthCol);
/*     */     }
/*     */   }
/*     */   
/*     */   public static void createUniqueConstraint(Value value) {
/* 351 */     Iterator iter = value.getColumnIterator();
/* 352 */     ArrayList cols = new ArrayList();
/* 353 */     while (iter.hasNext()) {
/* 354 */       cols.add(iter.next());
/*     */     }
/* 356 */     value.getTable().createUniqueKey(cols);
/*     */   }
/*     */   
/*     */   public static void addIndexes(Table hibTable, Index[] indexes, ExtendedMappings mappings) {
/* 360 */     for (Index index : indexes)
/*     */     {
/* 362 */       mappings.addSecondPass(new IndexOrUniqueKeySecondPass(hibTable, index.name(), index.columnNames(), mappings));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public static List<String[]> buildUniqueConstraints(UniqueConstraint[] constraintsArray)
/*     */   {
/* 369 */     List<String[]> result = new ArrayList();
/* 370 */     if (constraintsArray.length != 0) {
/* 371 */       for (UniqueConstraint uc : constraintsArray) {
/* 372 */         result.add(uc.columnNames());
/*     */       }
/*     */     }
/* 375 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void setDefaultName(String ownerEntity, String ownerEntityTable, String associatedEntity, String associatedEntityTable, String propertyName)
/*     */   {
/* 382 */     this.ownerEntity = ownerEntity;
/* 383 */     this.ownerEntityTable = ownerEntityTable;
/* 384 */     this.associatedEntity = associatedEntity;
/* 385 */     this.associatedEntityTable = associatedEntityTable;
/* 386 */     this.propertyName = propertyName;
/* 387 */     this.name = null;
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\annotations\TableBinder.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */