/*    */ package org.hibernate.cfg.annotations.reflection;
/*    */ 
/*    */ import java.lang.reflect.AnnotatedElement;
/*    */ import java.util.ArrayList;
/*    */ import java.util.HashMap;
/*    */ import java.util.List;
/*    */ import java.util.Map;
/*    */ import javax.persistence.EntityListeners;
/*    */ import javax.persistence.NamedNativeQuery;
/*    */ import javax.persistence.NamedQuery;
/*    */ import javax.persistence.SequenceGenerator;
/*    */ import javax.persistence.SqlResultSetMapping;
/*    */ import javax.persistence.TableGenerator;
/*    */ import org.dom4j.Element;
/*    */ import org.hibernate.annotations.common.reflection.AnnotationReader;
/*    */ import org.hibernate.annotations.common.reflection.java.JavaReflectionManager;
/*    */ import org.hibernate.util.ReflectHelper;
/*    */ 
/*    */ public class EJB3ReflectionManager
/*    */   extends JavaReflectionManager
/*    */ {
/* 22 */   private XMLContext xmlContext = new XMLContext();
/* 23 */   private HashMap defaults = null;
/*    */   
/*    */   public AnnotationReader buildAnnotationReader(AnnotatedElement annotatedElement) {
/* 26 */     if (this.xmlContext.hasContext()) {
/* 27 */       return new EJB3OverridenAnnotationReader(annotatedElement, this.xmlContext);
/*    */     }
/*    */     
/* 30 */     return super.buildAnnotationReader(annotatedElement);
/*    */   }
/*    */   
/*    */   public Map getDefaults() {
/*    */     XMLContext.Default xmlDefaults;
/* 35 */     if (this.defaults == null) {
/* 36 */       this.defaults = new HashMap();
/* 37 */       xmlDefaults = this.xmlContext.getDefault(null);
/* 38 */       List<Class> entityListeners = new ArrayList();
/* 39 */       for (String className : this.xmlContext.getDefaultEntityListeners()) {
/*    */         try {
/* 41 */           entityListeners.add(ReflectHelper.classForName(className, getClass()));
/*    */         }
/*    */         catch (ClassNotFoundException e) {
/* 44 */           throw new IllegalStateException("Default entity listener class not found: " + className);
/*    */         }
/*    */       }
/* 47 */       this.defaults.put(EntityListeners.class, entityListeners);
/* 48 */       for (Element element : this.xmlContext.getAllDocuments())
/*    */       {
/* 50 */         List<Element> elements = element.elements("sequence-generator");
/* 51 */         List<SequenceGenerator> sequenceGenerators = (List)this.defaults.get(SequenceGenerator.class);
/* 52 */         if (sequenceGenerators == null) {
/* 53 */           sequenceGenerators = new ArrayList();
/* 54 */           this.defaults.put(SequenceGenerator.class, sequenceGenerators);
/*    */         }
/* 56 */         for (Element subelement : elements) {
/* 57 */           sequenceGenerators.add(EJB3OverridenAnnotationReader.buildSequenceGeneratorAnnotation(subelement));
/*    */         }
/*    */         
/* 60 */         elements = element.elements("table-generator");
/* 61 */         List<TableGenerator> tableGenerators = (List)this.defaults.get(TableGenerator.class);
/* 62 */         if (tableGenerators == null) {
/* 63 */           tableGenerators = new ArrayList();
/* 64 */           this.defaults.put(TableGenerator.class, tableGenerators);
/*    */         }
/* 66 */         for (Element subelement : elements) {
/* 67 */           tableGenerators.add(EJB3OverridenAnnotationReader.buildTableGeneratorAnnotation(subelement, xmlDefaults));
/*    */         }
/*    */         
/* 70 */         List<NamedQuery> namedQueries = (List)this.defaults.get(NamedQuery.class);
/* 71 */         if (namedQueries == null) {
/* 72 */           namedQueries = new ArrayList();
/* 73 */           this.defaults.put(NamedQuery.class, namedQueries);
/*    */         }
/* 75 */         List<NamedQuery> currentNamedQueries = EJB3OverridenAnnotationReader.buildNamedQueries(element, false, xmlDefaults);
/* 76 */         namedQueries.addAll(currentNamedQueries);
/*    */         
/* 78 */         List<NamedNativeQuery> namedNativeQueries = (List)this.defaults.get(NamedNativeQuery.class);
/* 79 */         if (namedNativeQueries == null) {
/* 80 */           namedNativeQueries = new ArrayList();
/* 81 */           this.defaults.put(NamedNativeQuery.class, namedNativeQueries);
/*    */         }
/* 83 */         List<NamedNativeQuery> currentNamedNativeQueries = EJB3OverridenAnnotationReader.buildNamedQueries(element, true, xmlDefaults);
/* 84 */         namedNativeQueries.addAll(currentNamedNativeQueries);
/*    */         
/* 86 */         List<SqlResultSetMapping> sqlResultSetMappings = (List)this.defaults.get(SqlResultSetMapping.class);
/* 87 */         if (sqlResultSetMappings == null) {
/* 88 */           sqlResultSetMappings = new ArrayList();
/* 89 */           this.defaults.put(SqlResultSetMapping.class, sqlResultSetMappings);
/*    */         }
/* 91 */         List<SqlResultSetMapping> currentSqlResultSetMappings = EJB3OverridenAnnotationReader.buildSqlResultsetMappings(element, xmlDefaults);
/* 92 */         sqlResultSetMappings.addAll(currentSqlResultSetMappings);
/*    */       }
/*    */     }
/* 95 */     return this.defaults;
/*    */   }
/*    */   
/*    */   public XMLContext getXMLContext() {
/* 99 */     return this.xmlContext;
/*    */   }
/*    */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\annotations\reflection\EJB3ReflectionManager.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */