/*     */ package org.hibernate.type;
/*     */ 
/*     */ import java.io.ByteArrayInputStream;
/*     */ import java.io.Serializable;
/*     */ import java.sql.Blob;
/*     */ import java.sql.PreparedStatement;
/*     */ import java.sql.ResultSet;
/*     */ import java.sql.SQLException;
/*     */ import java.util.Map;
/*     */ import java.util.Properties;
/*     */ import org.dom4j.Node;
/*     */ import org.hibernate.EntityMode;
/*     */ import org.hibernate.HibernateException;
/*     */ import org.hibernate.MappingException;
/*     */ import org.hibernate.dialect.Dialect;
/*     */ import org.hibernate.engine.Mapping;
/*     */ import org.hibernate.engine.SessionFactoryImplementor;
/*     */ import org.hibernate.engine.SessionImplementor;
/*     */ import org.hibernate.lob.BlobImpl;
/*     */ import org.hibernate.usertype.ParameterizedType;
/*     */ import org.hibernate.util.ReflectHelper;
/*     */ import org.hibernate.util.SerializationHelper;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SerializableToBlobType
/*     */   extends AbstractLobType
/*     */   implements ParameterizedType
/*     */ {
/*     */   public static final String CLASS_NAME = "classname";
/*     */   private Class serializableClass;
/*     */   private SerializableType type;
/*     */   
/*     */   public int[] sqlTypes(Mapping mapping)
/*     */     throws MappingException
/*     */   {
/*  38 */     return new int[] { 2004 };
/*     */   }
/*     */   
/*     */   public Class getReturnedClass() {
/*  42 */     return this.serializableClass;
/*     */   }
/*     */   
/*     */   public boolean isEqual(Object x, Object y, EntityMode entityMode, SessionFactoryImplementor factory)
/*     */   {
/*  47 */     return this.type.isEqual(x, y);
/*     */   }
/*     */   
/*     */ 
/*     */   public int getHashCode(Object x, EntityMode entityMode, SessionFactoryImplementor session)
/*     */   {
/*  53 */     return this.type.getHashCode(x, null);
/*     */   }
/*     */   
/*     */   public Object get(ResultSet rs, String name) throws SQLException {
/*  57 */     Blob blob = rs.getBlob(name);
/*  58 */     if (rs.wasNull()) return null;
/*  59 */     int length = (int)blob.length();
/*  60 */     byte[] primaryResult = blob.getBytes(1L, length);
/*  61 */     return fromBytes(primaryResult);
/*     */   }
/*     */   
/*     */   private static byte[] toBytes(Object object) throws SerializationException {
/*  65 */     return SerializationHelper.serialize((Serializable)object);
/*     */   }
/*     */   
/*     */   private static Object fromBytes(byte[] bytes) throws SerializationException {
/*  69 */     return SerializationHelper.deserialize(bytes);
/*     */   }
/*     */   
/*     */   public void set(PreparedStatement st, Object value, int index, SessionImplementor session) throws SQLException {
/*  73 */     if (value != null)
/*     */     {
/*  75 */       byte[] toSet = toBytes(value);
/*  76 */       if (session.getFactory().getDialect().useInputStreamToInsertBlob()) {
/*  77 */         st.setBinaryStream(index, new ByteArrayInputStream(toSet), toSet.length);
/*     */       }
/*     */       else {
/*  80 */         st.setBlob(index, new BlobImpl(toSet));
/*     */       }
/*     */     }
/*     */     else {
/*  84 */       st.setNull(index, sqlTypes(null)[0]);
/*     */     }
/*     */   }
/*     */   
/*     */   public void setToXMLNode(Node node, Object value, SessionFactoryImplementor factory) throws HibernateException {
/*  89 */     this.type.setToXMLNode(node, value, factory);
/*     */   }
/*     */   
/*     */   public String toLoggableString(Object value, SessionFactoryImplementor factory) throws HibernateException {
/*  93 */     return this.type.toLoggableString(value, factory);
/*     */   }
/*     */   
/*     */   public Object fromXMLNode(Node xml, Mapping factory) throws HibernateException {
/*  97 */     return this.type.fromXMLNode(xml, factory);
/*     */   }
/*     */   
/*     */   public Object deepCopy(Object value, EntityMode entityMode, SessionFactoryImplementor factory) throws HibernateException
/*     */   {
/* 102 */     return this.type.deepCopy(value, null, null);
/*     */   }
/*     */   
/*     */   public boolean isMutable() {
/* 106 */     return this.type.isMutable();
/*     */   }
/*     */   
/*     */   public Object replace(Object original, Object target, SessionImplementor session, Object owner, Map copyCache) throws HibernateException
/*     */   {
/* 111 */     return this.type.replace(original, target, session, owner, copyCache);
/*     */   }
/*     */   
/*     */   public boolean[] toColumnNullness(Object value, Mapping mapping) {
/* 115 */     return this.type.toColumnNullness(value, mapping);
/*     */   }
/*     */   
/*     */   public void setParameterValues(Properties parameters) {
/* 119 */     if (parameters != null) {
/* 120 */       String className = parameters.getProperty("classname");
/* 121 */       if (className == null) {
/* 122 */         throw new MappingException("No class name defined for type: " + SerializableToBlobType.class.getName());
/*     */       }
/*     */       
/*     */       try
/*     */       {
/* 127 */         this.serializableClass = ReflectHelper.classForName(className);
/*     */       }
/*     */       catch (ClassNotFoundException e) {
/* 130 */         throw new MappingException("Unable to load class from classname parameter", e);
/*     */       }
/*     */     }
/* 133 */     this.type = new SerializableType(this.serializableClass);
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\type\SerializableToBlobType.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */