package com.netapp.myasup.jasper.util;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.OutputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class FileManager {

	private final Log LOG = LogFactory.getLog(getClass());
	
	public void writeFile(Object object, String serFileName) {

		try {
			// use buffering
			OutputStream file = new FileOutputStream(serFileName);
			OutputStream buffer = new BufferedOutputStream(file);
			ObjectOutput output = new ObjectOutputStream(buffer);
			try {
				output.writeObject(object);
			} finally {
				output.close();
			}
		} catch (IOException ex) {
			// fLogger.log(Level.SEVERE, "Cannot perform output.", ex);
		}
	}

	public String createFolders(String rootDir ,String folderName) {
		String finalPath = new StringBuilder().append(rootDir).append(File.separator).append(folderName).toString();
		File path = new File(finalPath);
		if (!path.exists()) {
			if (!path.mkdirs()) {
				String msg = "Failed to create folder path: " + path.getAbsolutePath();
				LOG.error(msg);
//				throw new WillowsExceptison(msg);
			}
		} else {
			if (LOG.isDebugEnabled())
				LOG.debug(finalPath + " already exists");
		}
		LOG.info("FolderCreated "+finalPath);
		return finalPath;

	}
	
	public Object readFile(String serFileName) {

		Object object = null;
		try {
			// use buffering
//			InputStream file = new FileInputStream();
//			InputStream buffer = new BufferedInputStream(new ClassPathResource(serFileName).getInputStream());
//			InputStream buffer = new BufferedInputStream(getClass().getClassLoader().getResourceAsStream(serFileName));
			InputStream buffer = this.getClass().getClassLoader().getResourceAsStream(serFileName); 
			LOG.info("Input Stream "+buffer);
			ObjectInput input = new ObjectInputStream(buffer);
			try {
				// deserialize the List
				object = (Object) input.readObject();
				// display its data

			} finally {
				input.close();
			}
		} catch (ClassNotFoundException ex) {
			// fLogger.log(Level.SEVERE,
			// "Cannot perform input. Class not found.", ex);
		} catch (IOException ex) {
			// fLogger.log(Level.SEVERE, "Cannot perform input.", ex);
		}
		LOG.info("Serialized Design Object "+object);
		return object;
	}

	public boolean createDirectory(File file) {
		// Delete Directory if alreday exists
		if (file.exists())
			deleteDirectory(file);
		boolean status = file.mkdirs();
		if (status)
			System.out.println(" Successfull of creating Directory " + file.getPath());
		return status;
	}

	public boolean deleteDirectory(File dir) {
		LOG.info("Folder Absolute Path " +dir.getAbsolutePath());
		if (dir.isDirectory()) {
			LOG.info("Is a Directory " +dir.getAbsolutePath());
			String[] children = dir.list();
			LOG.info("children length " +children.length);
			for (int i = 0; i < children.length; i++) {
				File delFile = new File(dir, children[i]);
				if (!delFile.exists()) {
					LOG.info("Cannot find directory to delete" + delFile.getPath());
					return false;
				}
				boolean success = deleteDirectory(delFile);
				LOG.info(delFile + ": success? " + success);
				if (!success) {
					LOG.info("failure during delete directory" + delFile.getPath());
					return false;
				}
			}
		}
		boolean deleteResult = dir.delete();
		// The directory is now empty so now it can be smoked
		LOG.info("deleteDirectory "+dir+ " "+deleteResult);
		return deleteResult;
	}
	
	public static File createTempDir(String dirName) throws IOException
	{
	    final File sysTempDir = new File(System.getProperty("java.io.tmpdir"));
	    File newTempDir;
	    final int maxAttempts = 9;
	    int attemptCount = 0;
	    do
	    {
	        attemptCount++;
	        if(attemptCount > maxAttempts)
	        {
	            throw new IOException(
	                    "The highly improbable has occurred! Failed to " +
	                    "create a unique temporary directory after " +
	                    maxAttempts + " attempts.");
	        }
	        newTempDir = new File(sysTempDir, dirName);
	    } while(newTempDir.exists());

	    if(newTempDir.mkdirs())
	    {
	        return newTempDir;
	    }
	    else
	    {
	        throw new IOException(
	                "Failed to create temp dir named " +
	                newTempDir.getAbsolutePath());
	    }
	}

	/**
	 * Recursively delete file or directory
	 * @param fileOrDir
	 *          the file or dir to delete
	 * @return
	 *          true iff all files are successfully deleted
	 */
	
//	public static boolean recursiveDelete(File fileOrDir)
//	{
//	    if(fileOrDir.isDirectory())
//	    {
//	        // recursively delete contents
//	        for(File innerFile: fileOrDir.listFiles())
//	        {
//	            if(!FileUtilities.recursiveDelete(innerFile))
//	            {
//	                return false;
//	            }
//	        }
//	    }
//
//	    return fileOrDir.delete();
//	}

}
