package com.netapp.myasup.reports.urp.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.bind.JAXBContext;

import org.apache.commons.io.IOUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.HttpResponseException;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.netapp.myasup.reports.exception.ReportCreationException;
import com.netapp.myasup.reports.urp.bean.pojo.ClientResource;
import com.netapp.myasup.reports.urp.bean.pojo.JobIdList;
import com.netapp.myasup.reports.urp.bean.pojo.JobState;
import com.netapp.myasup.reports.urp.bean.pojo.Jobs;
import com.netapp.myasup.reports.urp.bean.pojo.Jobsummary;
import com.netapp.myasup.reports.urp.bean.pojo.ResourceLookup;


public class JasperRestUtil {

	static final Logger logger = Logger.getLogger(JasperRestUtil.class);
	static final String DATE_FORMAT 			= "yyyy-MM-dd'T'HH:mm:ss";	
	static final String RESOURCE_TYPE_FOLDER 	= "folder";
	static final String RESOURCE_TYPE_FILE 		= "file";

	//private JasperserverRestClient jasperserverRestClient;

	//private Session session = null;
	
	
    private String host;
    private String port;
    private String resource;
	private String username;
	private String password;
	
	

	/*private Session getSession(){
		
		if (session == null || session.getStorage() == null || session.getStorage().getSessionId() == null) {
			session = jasperserverRestClient.authenticate(username, password);
		}
		
		return session;
	}
	


	public void setJasperserverRestClient(
			JasperserverRestClient jasperserverRestClient) {
		this.jasperserverRestClient = jasperserverRestClient;
	}
	*/

	public void setUsername(String username) {
		this.username = username;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public void setHost(String host) {
		this.host = host;
	}

	public void setPort(String port) {
		this.port = port;
	}

	public void setResource(String resource) {
		this.resource = resource;
	}

	private static long getTime() {
		return System.currentTimeMillis();
	}

	private static long getTimeDiff(long startTime) {
		return System.currentTimeMillis() - startTime;
	}

	
	//same methid can be used for both on demand and generate report

	public List<com.netapp.myasup.reports.urp.bean.pojo.Job> createReport(List<com.netapp.myasup.reports.urp.bean.pojo.Job> jobs)

	throws ReportCreationException {

		logger.info("createReport ");

		List<com.netapp.myasup.reports.urp.bean.pojo.Job> successfulJobs = new ArrayList<com.netapp.myasup.reports.urp.bean.pojo.Job>();

		CloseableHttpClient httpClient = getHttpClient();

		// URIBuilder builder;

		HttpPut httpput = null;

		try {

			//URIBuilder builder = new URIBuilder("http://vmwasmrep01-dev:8080/");
			URIBuilder builder = new URIBuilder("http://"+ host + ":"+ port + "/");

			builder.setPath("/jasperserver-pro/rest_v2/jobs");

			httpput = new HttpPut(builder.build());

			httpput.setHeader("Accept", "application/json");

			httpput.setHeader("Content-type", "application/json");

		} catch (URISyntaxException e) {

			e.printStackTrace();

			throw new ReportCreationException(e.getMessage());

		}

		Gson gson = new Gson();

		for (com.netapp.myasup.reports.urp.bean.pojo.Job job : jobs) {

			try {

				String jobJson = gson.toJson(job);

				// System.out.println(jobJson);

				StringEntity entity = null;

				try {

					entity = new StringEntity(jobJson);

					httpput.setEntity(entity);

				} catch (UnsupportedEncodingException e1) {

					throw new ReportCreationException(e1.getMessage());

				}

				// Create a custom response handler

				ResponseHandler<String> responseHandler = new ResponseHandler<String>() {

					public String handleResponse(final HttpResponse response)

					throws ClientProtocolException, IOException {

						int status = response.getStatusLine().getStatusCode();

						if (status >= 200 && status < 300) {

							HttpEntity entity = response.getEntity();

							return entity != null ? EntityUtils

							.toString(entity) : null;

						} else {

							throw new ClientProtocolException(

							"Unexpected response status: " + status);

						}

					}

				};

				try {

					String response = httpClient.execute(httpput,

					responseHandler);

					Gson responseGson = new GsonBuilder().setDateFormat(DATE_FORMAT).create();

					com.netapp.myasup.reports.urp.bean.pojo.Job val = responseGson.fromJson(response, com.netapp.myasup.reports.urp.bean.pojo.Job.class);

					successfulJobs.add(val);

				} catch (IOException e) {

					throw new ReportCreationException(e.getMessage());

				}

			} catch (ReportCreationException e) {

				logger.info("exception in processing message" + job.getId());

				logger.error(e.getMessage());

			}

		}

		try {

			httpClient.close();

		} catch (IOException e) {

			// TODO Auto-generated catch block

			e.printStackTrace();

		}

		return successfulJobs;

	}
	
	public boolean deleteResources(Collection<String> reportLocns) throws Exception{
		CloseableHttpClient httpClient = getHttpClient();
		URIBuilder builder;
		try {				
			logger.info("deleting resource ");
			builder = new URIBuilder("http://"+ host + ":"+ port + "/");
			builder.setPath(resource);
			
			if(reportLocns == null || reportLocns.size()==0){
				throw new Exception("No resource to delete specified.");
			}
			
			for (String locn : reportLocns) {
				builder.addParameter("resourceUri", locn);
			}
			
			HttpDelete httpDel = new HttpDelete(builder.build());

			ResponseHandler<Boolean> responseHandler = new ResponseHandler<Boolean>() {
				public Boolean handleResponse(final HttpResponse response)
						throws ClientProtocolException, IOException {
					
					int status = response.getStatusLine().getStatusCode();
					if (status == 204 || status == 404) {//No Content or if Not Found
						return true;
					} else {
						logger.error("Unexpected response status while deleting resource: "+ status);
						return false;
					}
				}
			};

			Boolean response = httpClient.execute(httpDel, responseHandler);	
			return response.booleanValue();
			
		} catch (URISyntaxException e) {
			logger.error("Error deleting resource:", e);
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			logger.error("Error deleting resource:", e);
			e.printStackTrace();
		} 
		return false;
	}

	//batch delete of resources
	public List<String> deleteResource(Map<String, String> reportLocns) {
		
		Set<String> jobIds = reportLocns.keySet();
		//List<String> failedJobs = Arrays.asList(jobIds.toArray(new String[jobIds.size()]));
		List<String> failedJobs = new ArrayList<String>();
			
		try {				
			boolean deleted = deleteResources(reportLocns.values());
			
			if(deleted){
				logger.info("deleted resource " +":"+deleted);
			} else {
				logger.error("Error deleting resource:");
				//failedJobs = new ArrayList<String>();
				if(jobIds.size() == 1){
					//only one resource was to be deleted and it failed
					failedJobs.add(jobIds.iterator().next());
				} else {
					//for batch delete find the resources which were not deleted.
					Map<String, ResourceLookup> resources = getALLJobState(null, false);
					if(resources != null){
						for(String jobId : jobIds){
							if(resources.containsKey(reportLocns.get(jobId))){
								//resource still exist in the jasper server
								logger.error("delete failed for resource: "+reportLocns.get(jobId));
								failedJobs.add(jobId);
							}
						}
					}
				}
			}			
		} catch (Exception e) {
			logger.error("Error deleting resource:", e);
			e.printStackTrace();
		}
		return failedJobs;
	}

	public String createResource(String userFldr, String jobFldr) {

		CloseableHttpClient httpClient = getHttpClient();
		URIBuilder builder;
		try {
			
			builder = new URIBuilder("http://"+ host + ":"+ port + "/");
			builder.setPath("/jasperserver-pro/rest_v2/resources/MyAsup/reports/Output/"+userFldr+"/");
			builder.addParameter("createFolders", "true");
			
			ResourceLookup resourceLookup = new ResourceLookup();
			resourceLookup.setLabel(jobFldr);
			resourceLookup.setDescription(jobFldr);
			resourceLookup.setResourceType("folder");

			Gson gson = new Gson();
			String jsonString = gson.toJson(resourceLookup);
			StringEntity entity = new StringEntity(jsonString);
			HttpPost httpPost = new HttpPost(builder.build());
			httpPost.setHeader("Accept", "application/json");
			httpPost.setHeader("Content-type",
					"application/repository.folder+json");
			httpPost.setEntity(entity);

			ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
				public String handleResponse(final HttpResponse response)
						throws ClientProtocolException, IOException {
					
					int status = response.getStatusLine().getStatusCode();
					if (status >= 200 && status < 300) {
						HttpEntity entity = response.getEntity();
						return entity != null ? EntityUtils.toString(entity) : null;

					} else {
						throw new ClientProtocolException(
						"Unexpected response status: " + status);
					}
				}
			};

			String response = httpClient.execute(httpPost, responseHandler);
			Gson responseGson = new GsonBuilder().setDateFormat(DATE_FORMAT).create();
			ResourceLookup val = responseGson.fromJson(response,
					ResourceLookup.class);

			return val.getUri();

		} catch (URISyntaxException e) {
			logger.error("Error creating resource:"+userFldr+":"+jobFldr, e);
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			logger.error("Error creating resource:"+userFldr+":"+jobFldr, e);
			e.printStackTrace();
		} catch (IOException e) {
			logger.error("Error creating resource:"+userFldr+":"+jobFldr, e);
			e.printStackTrace();
		}

		return null;
	}
	
	
	private CloseableHttpClient getHttpClient(){
		CredentialsProvider credsProvider = new BasicCredentialsProvider();
		credsProvider.setCredentials(new AuthScope(host, Integer.parseInt(port)),new UsernamePasswordCredentials(username, password));
		CloseableHttpClient httpclient = HttpClients.custom().setDefaultCredentialsProvider(credsProvider).build();
		return  httpclient;
		
	}
	
	
	
	private ResourceLookup clientResourceToDownload(String reportLocation, String reportLabel) {
		CloseableHttpClient httpclient = getHttpClient();
		try {
			URIBuilder builder = new URIBuilder("http://"+ host + ":"+ port + "/");
			builder.setPath(resource);
			builder.setParameter("limit", "10")
				.setParameter("sortBy", "creationDate")
				.setParameter("folderUri", reportLocation)
				.setParameter("recursive", "false");

			HttpGet httpget = new HttpGet(builder.build());
			
			//System.out.println(Base64.class.getProtectionDomain().getCodeSource().getLocation());

			System.out.println("Executing request " + httpget.getRequestLine());

			httpget.setHeader("Accept", "application/json");
			httpget.setHeader("Content-type", "application/json");

			// Create a custom response handler
			ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
				public String handleResponse(final HttpResponse response)
						throws ClientProtocolException, IOException {
					int status = response.getStatusLine().getStatusCode();
					if (status >= 200 && status < 300) {
						HttpEntity entity = response.getEntity();

						return entity != null ? EntityUtils.toString(entity)
								: null;
					} else {
						throw new ClientProtocolException(
								"Unexpected response status: " + status);
					}
				}
			};

			String response = httpclient.execute(httpget, responseHandler);

			Gson gson = new GsonBuilder().setDateFormat(DATE_FORMAT).create();

			ClientResource clientResource = gson.fromJson(response,ClientResource.class);

			ResourceLookup[] clientResourceLookups = clientResource
					.getResourceLookup();
			ResourceLookup resourceLookup = null;
			String latestCreationDate = null;

			for (ResourceLookup clientResourceLookup : clientResourceLookups) {
				if (clientResourceLookup.getLabel().startsWith(reportLabel)) {
					if (resourceLookup == null) {

						latestCreationDate = clientResourceLookup
								.getCreationDate();
						resourceLookup = clientResourceLookup;

					} else {
						if (clientResourceLookup.getCreationDate().compareTo(
								latestCreationDate) > 0) {

							latestCreationDate = clientResourceLookup
									.getCreationDate();

							resourceLookup = clientResourceLookup;

						}
					}
				}
			}
			return resourceLookup;

		} catch (URISyntaxException e) {
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;

	}
	
	private File saveReportToFile(ResourceLookup clientResource, String outputPath) {
		CloseableHttpClient httpclient = getHttpClient();
		File targetFile = null;
		try {
			//URIBuilder builder = new URIBuilder("http://vmwasmrep01-dev:8080/");
			URIBuilder builder = new URIBuilder("http://"+ host + ":"+ port + "/");
			builder.setPath("/jasperserver-pro/rest_v2/resources" + clientResource.getUri());
			HttpGet httpget = new HttpGet(builder.build());
			CloseableHttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();

			if (entity != null) {
				InputStream inputStream = entity.getContent();

				String finalOutputPath = outputPath + File.separator+ clientResource.getLabel();

				targetFile = new File(finalOutputPath);
				OutputStream outStream = null;
				try {
					outStream = new FileOutputStream(targetFile);
					IOUtils.copy(inputStream, outStream);
				} catch (FileNotFoundException e) {
					e.printStackTrace();
					return null;
				} catch (IOException e) {
					e.printStackTrace();
					return null;
				} finally {
					try {
						inputStream.close();
						outStream.close();
					} catch (IOException e) {
						e.printStackTrace();
						return null;
					}
				}
			}
		} catch (URISyntaxException e1) {
			e1.printStackTrace();
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return targetFile;

	}
	
	public File downloadReport(String reportLocation , String reportLabel, String outputPath) throws ClientProtocolException, IOException {
		ResourceLookup clientResource = clientResourceToDownload( reportLocation, reportLabel);
		return saveReportToFile( clientResource , outputPath);
	}
	

/*	public  Job fetchJobDetails(long requestId)
			 {
		
		OperationResult<Job> result = getSession().jobsService().job(requestId).get();
		
		return result.getEntity();
		

	}*/
	
	
	public com.netapp.myasup.reports.urp.bean.pojo.JobState getJobState(
			long requestId) {

		CloseableHttpClient httpClient = getHttpClient();
		URIBuilder builder;

		try {

			builder = new URIBuilder("http://" + host + ":" + port + "/");
			builder.setPath("/jasperserver-pro/rest_v2/jobs/" + requestId
					+ "/state");

			HttpGet httpGet = new HttpGet(builder.build());
			httpGet.setHeader("Accept", "application/json");

			ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
				public String handleResponse(final HttpResponse response)
						throws ClientProtocolException, IOException,
						HttpResponseException {

					int status = response.getStatusLine().getStatusCode();

					if (status >= 200 && status < 300) {

						HttpEntity entity = response.getEntity();

						return entity != null ? EntityUtils.toString(entity)
								: null;

					} else {
						throw new HttpResponseException(status,
								"Unexpected response status: " + status);
					}
				}
			};

			String response = httpClient.execute(httpGet, responseHandler);

			Gson responseGson = new GsonBuilder().setDateFormat(DATE_FORMAT)
					.create();

			com.netapp.myasup.reports.urp.bean.pojo.JobState val = responseGson
					.fromJson(
							response,
							com.netapp.myasup.reports.urp.bean.pojo.JobState.class);

			return val;

		} catch (Exception e) {
			// e.printStackTrace();
			logger.info("resource not found for job id" + requestId);
			return null;

		}

	}
	
	
	public boolean deleteJob(long requestId) {
		
		
		CloseableHttpClient httpClient = getHttpClient();
		URIBuilder builder;

		try {

			builder = new URIBuilder("http://" + host + ":" + port + "/");
			builder.setPath("/jasperserver-pro/rest_v2/jobs/"+requestId);

			HttpDelete httpDelete = new HttpDelete(builder.build());
			httpDelete.setHeader("Accept", "application/json");
			


			ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
				public String handleResponse(final HttpResponse response)
						throws ClientProtocolException, IOException,
						HttpResponseException {

					int status = response.getStatusLine().getStatusCode();

					if ((status >= 200 && status < 300) || status == 404) {
						
						HttpEntity entity = response.getEntity();
						return entity != null ? EntityUtils.toString(entity)
								: null;

					} else {
						throw new HttpResponseException(status,
								"Unexpected response status: " + status);
					}
				}
			};

			String response = httpClient.execute(httpDelete, responseHandler);

		} catch (Exception e) {
			e.printStackTrace();
			return false;

		}

		return true;
		
		
		
		//OperationResult<Integer> result = getSession().jobsService().job(requestId).delete();

		//return result.getEntity();
		
		
	}
	
	public boolean pauseJob(String[] jobArray)  {

		CloseableHttpClient httpClient = getHttpClient();
		URIBuilder builder;

		try {

			builder = new URIBuilder("http://" + host + ":" + port + "/");
			builder.setPath("/jasperserver-pro/rest_v2/jobs/pause");

			HttpPost httpPost = new HttpPost(builder.build());
			httpPost.setHeader("Content-type", "application/xml");
			

			JobIdList jobIdList = new JobIdList();
			jobIdList.setJobId(jobArray);

			StringWriter sw = new StringWriter();

			JAXBContext context = JAXBContext.newInstance(JobIdList.class);
			
			context.createMarshaller().marshal(jobIdList, sw);

			StringEntity entity = new StringEntity(sw.toString());
			httpPost.setEntity(entity);

			ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
				public String handleResponse(final HttpResponse response)
						throws ClientProtocolException, IOException,
						HttpResponseException {

					int status = response.getStatusLine().getStatusCode();

					if (status >= 200 && status < 300) {
						
						HttpEntity entity = response.getEntity();
						return entity != null ? EntityUtils.toString(entity)
								: null;

					} else {
						throw new HttpResponseException(status,
								"Unexpected response status: " + status);
					}
				}
			};

			String response = httpClient.execute(httpPost, responseHandler);

		} catch (Exception e) {
			e.printStackTrace();
			return false;

		}

		return true;

	}
	
	
	public boolean resumeJob(String[] jobArray) {

		CloseableHttpClient httpClient = getHttpClient();
		URIBuilder builder;

		try {

			builder = new URIBuilder("http://" + host + ":" + port + "/");
			builder.setPath("/jasperserver-pro/rest_v2/jobs/resume");

			HttpPost httpPost = new HttpPost(builder.build());
			httpPost.setHeader("Content-type", "application/xml");
			

			JobIdList jobIdList = new JobIdList();
			jobIdList.setJobId(jobArray);

			StringWriter sw = new StringWriter();

			JAXBContext context = JAXBContext.newInstance(JobIdList.class);
			
			context.createMarshaller().marshal(jobIdList, sw);

			StringEntity entity = new StringEntity(sw.toString());
			httpPost.setEntity(entity);

			ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
				public String handleResponse(final HttpResponse response)
						throws ClientProtocolException, IOException,
						HttpResponseException {

					int status = response.getStatusLine().getStatusCode();

					if (status >= 200 && status < 300) {
						
						HttpEntity entity = response.getEntity();
						return entity != null ? EntityUtils.toString(entity)
								: null;

					} else {
						throw new HttpResponseException(status,
								"Unexpected response status: " + status);
					}
				}
			};

			String response = httpClient.execute(httpPost, responseHandler);

		} catch (Exception e) {
			e.printStackTrace();
			return false;

		}

		return true;

	}
	
	public Map<String, ResourceLookup> getALLJobState(String userFldr, boolean file) throws Exception{
		CloseableHttpClient httpClient = getHttpClient();		
		URIBuilder builder;

		Map<String, ResourceLookup> resourceMap = null;
		String resourceType = file ? RESOURCE_TYPE_FILE : RESOURCE_TYPE_FOLDER;
		try {

			String folderUri = ("".equalsIgnoreCase(userFldr) || userFldr == null) ? 
					"/MyAsup/reports/Output" : "/MyAsup/reports/Output/"+userFldr;
			builder = new URIBuilder("http://"+ host + ":"+ port + "/");
			builder.setPath(resource);			
			builder.addParameter("folderUri", folderUri);
			//To return all results without pagination add limit=0
			//https://support.jaspersoft.com/documentation/jasperreports-server/601
			//up to 1000 records are retrieved under a second now, if required we can add pagination later 
			builder.addParameter("limit", "0");
			//get only the required resource types
			//e.g. for statistics get only file type; for batch delete get only folder type
			builder.addParameter("type", resourceType);
			
			HttpGet httpGet = new HttpGet(builder.build());
			httpGet.setHeader("Accept", "application/json");

			ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
				public String handleResponse(final HttpResponse response)
						throws ClientProtocolException, IOException, HttpResponseException {

					int status = response.getStatusLine().getStatusCode();					
					if (status >= 200 && status < 300) {						
						if(status == 204){
							return "No Content";							
						}

						HttpEntity entity = response.getEntity();
						return entity != null ? EntityUtils.toString(entity) : null;

					} else {
						throw new HttpResponseException(status, "Unexpected response status: " + status);
					}
				}
			};

			
			String response = httpClient.execute(httpGet, responseHandler);			
			if(response != null){
				if(response.contains("No Content")){
					return null;
				} else {				
					
					Gson responseGson = new GsonBuilder().setDateFormat(DATE_FORMAT).create();
					ClientResource val = responseGson
							.fromJson(response, ClientResource.class);
					
					if(val != null){						
						resourceMap = new HashMap<String, ResourceLookup>();
						
						ResourceLookup[] resources = val.getResourceLookup();
						String tmp = "";
						for(ResourceLookup r:resources){
							//consider only files for subjob
							//response is not in a hierarchy structure; folders and files are listed in a  flat structure
							tmp = r.getUri();
/*							if(file && tmp.lastIndexOf("-")>0){
								//remove the timestamp information from file name
								//check if the filename contains the following pattern : -201506040539.
								if(tmp.subSequence(tmp.lastIndexOf("-"), tmp.lastIndexOf(".")).toString().matches("-\\d{12}")){
									tmp = tmp.replace(tmp.subSequence(tmp.lastIndexOf("-"), tmp.lastIndexOf(".")),"");
								}
							}*/ 
							resourceMap.put(tmp, r);						
						}
						return resourceMap;
					}
				}
			}			
		}catch(Exception e){
			logger.error("Error retriving resource details from "+userFldr,e );
			e.printStackTrace();
		}
		
		return null;
	}
	
	public Map<String, ResourceLookup> getALLJobState(String userFldr) throws Exception{
		return getALLJobState(userFldr, true);
	}
	
	
	public Map<Long, JobState> getALLJobState(Set<Long> jobIds) throws Exception{
		
		
		Map<Long, JobState> jStateMap = new HashMap<Long, JobState>();
		
		CloseableHttpClient httpClient = getHttpClient();		
		URIBuilder builder;

		try {

			builder = new URIBuilder("http://"+ host + ":"+ port + "/");
			builder.setPath("/jasperserver-pro/rest_v2/jobs");
			
			for (long jobId : jobIds) {
				
				builder.addParameter("id", String.valueOf(jobId));
			}
			
			
			HttpGet httpGet = new HttpGet(builder.build());
			httpGet.setHeader("Accept", "application/json");

			
			ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
				public String handleResponse(final HttpResponse response)
						throws ClientProtocolException, IOException, HttpResponseException {

					int status = response.getStatusLine().getStatusCode();
					
					if (status >= 200 && status < 300) {						
						if(status == 204){
							return "No Content";							
						}

						HttpEntity entity = response.getEntity();
						return entity != null ? EntityUtils.toString(entity) : null;

					} else {
						throw new HttpResponseException(status, "Unexpected response status: " + status);
					}
				}
			};

			String response = httpClient.execute(httpGet, responseHandler);			
			if (response != null) {

				Gson responseGson = new GsonBuilder()
						.setDateFormat(DATE_FORMAT).create();
				Jobs val = responseGson.fromJson(response, Jobs.class);

				for (Jobsummary jobsummary : val.getJobsummary()) {

					jStateMap.put(jobsummary.getId(), jobsummary.getState());

				}
			}
			
		}catch(Exception e){
			e.printStackTrace();
			logger.info("Not able to get the report statistics data");
		}
		
		return jStateMap;	
	
		
	}
	
	public Date getLastRunDate(String rootFolder, String filename){
		
		CloseableHttpClient httpClient = getHttpClient();		
		URIBuilder builder;

		try {

			builder = new URIBuilder("http://"+ host + ":"+ port + "/");
			builder.setPath(resource);			
			builder.addParameter("folderUri", rootFolder);
			HttpGet httpGet = new HttpGet(builder.build());
			httpGet.setHeader("Accept", "application/json");

			//httpGet.setHeader("Content-type", "application/json");			
			
			ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
				public String handleResponse(final HttpResponse response)
						throws ClientProtocolException, IOException, HttpResponseException {

					int status = response.getStatusLine().getStatusCode();
					
					if (status >= 200 && status < 300) {						
						if(status == 204){
							return "No Content";							
						}

						HttpEntity entity = response.getEntity();
						return entity != null ? EntityUtils.toString(entity) : null;

					} else {
						throw new HttpResponseException(status, "Unexpected response status: " + status);
					}
				}
			};

			String response = httpClient.execute(httpGet, responseHandler);			
			if(response != null){
				if(response.contains("No Content")){
					return null;
				} else {

					Gson responseGson = new GsonBuilder().setDateFormat(DATE_FORMAT).create();
					ClientResource val = responseGson
							.fromJson(response, ClientResource.class);
					
					if(val != null){
						ResourceLookup[] resources = val.getResourceLookup();
						for(ResourceLookup resource : resources){
							if(resource.getUri().contains(filename)){
								return resource.getUpdateDate();
							}
						}
						return null;
					}
				}
			}
			
		}catch(Exception e){
			e.printStackTrace();
		}
		
		return null;		
	}
	
	public ResourceLookup[] getJobStatus(String rootFolder){
		CloseableHttpClient httpClient = getHttpClient();		
		URIBuilder builder;

		try {

			builder = new URIBuilder("http://"+ host + ":"+ port + "/");
			builder.setPath(resource);			
			builder.addParameter("folderUri", rootFolder);
			HttpGet httpGet = new HttpGet(builder.build());
			httpGet.setHeader("Accept", "application/json");

			//httpGet.setHeader("Content-type", "application/json");			
			
			ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
				public String handleResponse(final HttpResponse response)
						throws ClientProtocolException, IOException, HttpResponseException {

					int status = response.getStatusLine().getStatusCode();
					
					if (status >= 200 && status < 300) {						
						if(status == 204){
							return "No Content";							
						}

						HttpEntity entity = response.getEntity();
						return entity != null ? EntityUtils.toString(entity) : null;

					} else {
						throw new HttpResponseException(status, "Unexpected response status: " + status);
					}
				}
			};

			String response = httpClient.execute(httpGet, responseHandler);			
			if(response != null){
				if(response.contains("No Content")){
					return null;
				} else {

					Gson responseGson = new GsonBuilder().setDateFormat(DATE_FORMAT).create();
					ClientResource val = responseGson
							.fromJson(response, ClientResource.class);
					
					if(val != null){
						ResourceLookup[] resources = val.getResourceLookup();
						return resources;
					}
				}
			}
			
		}catch(Exception e){
			logger.error("Error retriving resource details from "+rootFolder,e );
			e.printStackTrace();
		}
		
		return null;
	}	
	
	public boolean deleteOldFiles(String rootFolder, int retainLatestCopies) {
		ResourceLookup[] resources = getJobStatus(rootFolder);
		if(resources != null){
			List<String> reportNames = new ArrayList<String>();
			for (ResourceLookup res : resources) {
				reportNames.add(res.getUri());
			}
			
			logger.debug("Sorting report names in reverse order...");
			Collections.sort(reportNames, Collections.reverseOrder());
			logger.debug("Sorted report names...");
			String lastReportName = null;
			List<String> reportsToDelete = new ArrayList<String>();
			int numOfReportsFound = 0;
			
			for (String reportName : reportNames) {
				//Get the file prefix excluding timestamp
				String reportPrefix = reportName.substring(0, reportName.lastIndexOf("-"));
				logger.debug(reportName);
				if (lastReportName == null || !lastReportName.equalsIgnoreCase(reportPrefix)) {
					lastReportName = reportPrefix;
					numOfReportsFound = 1;
				}
				else {
					numOfReportsFound++;
					if (numOfReportsFound > retainLatestCopies) {
						reportsToDelete.add(reportName);
					}
				}		
			}
			boolean del = false;
			try {
				del = this.deleteResources(reportsToDelete);
				logger.info("delete files from folder:"+rootFolder+":success:"+del);
			} catch (Exception e) {
				logger.error("Error deleting resources from "+rootFolder, e);
			}
			return del;
		} else {
			return true;
		}
	}
	
	public static String convertSpecialChars(String input) {
		Pattern regex = Pattern.compile("[!%^*$&+,:;=?@#|-]");
		Matcher matcher = regex.matcher(input);
		while (matcher.find()) {
			String s = matcher.group();
			input = input.replaceAll("\\" + s, "_");
		}
		return input;
	}
	
	
	public static void main(String[] args) throws Exception {
		
		
		long startTime = getTime();
		
		
		JasperRestUtil util = new JasperRestUtil();
		
		//util.downloadReport("/MyAsup/reports/Output/velur_5120", "Customer_1031904_ars_detail", "/Users/kodiraj/Desktop");
		
//		try {
//			util.downloadReport("/MyAsup/reports/Output/velur_5120", "Customer_1031904_ars_detail", "c://");
//			//util.clientResourceToDownload2("/MyAsup/reports/Output/velur_5120", "");
//		} catch (ClientProtocolException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		} catch (IOException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
		
//		String response = "{\"resourceLookup\":[{\"version\":0,\"permissionMask\":1,\"creationDate\":\"2015-05-21T02:34:10\",\"updateDate\":\"2015-05-21T02:34:10\",\"label\":\"Customer_1003013_ars_detail.xlsx\",\"description\":\"Description\",\"uri\":\"/MyAsup/reports/Output/velur_5680/Customer_1003013_ars_detail.xlsx\",\"resourceType\":\"file\"},{\"version\":0,\"permissionMask\":1,\"creationDate\":\"2015-05-21T02:34:09\",\"updateDate\":\"2015-05-21T02:34:09\",\"label\":\"Customer_1003013_sysconfig.xlsx\",\"description\":\"Description\",\"uri\":\"/MyAsup/reports/Output/velur_5680/Customer_1003013_sysconfig.xlsx\",\"resourceType\":\"file\"}]}";
//		Gson responseGson = new GsonBuilder().setDateFormat(DATE_FORMAT).create();
//		com.netapp.myasup.reports.urp.bean.pojo.ClientResource val = responseGson
//				.fromJson(response,
//						com.netapp.myasup.reports.urp.bean.pojo.ClientResource.class);
//		System.out.println(val);
//		System.out.println(getTimeDiff(startTime)); 
		
		util.setHost("vmwasmrep01-dev.corp.netapp.com");
		util.setPort("8080");
		util.setPassword("superuser");
		util.setUsername("superuser");
		util.setResource("/jasperserver-pro/rest_v2/resources");
//		util.deleteResource("velur", Arrays.asList(6064L));
		Map<String, String> locn = new HashMap<String, String>();
		locn.put("6086","/MyAsup/reports/Output/rejwan/6086");
		locn.put("6131","/MyAsup/reports/Output/rrajasek/6131");
		locn.put("6132","/MyAsup/reports/Output/rrajasek/6132");
		util.deleteResource(locn);
//		util.createResource("velur", "rrajasek");
	}
	
	
	
	

}
