package com.netapp.myasup.reportRest.bean;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.SortedSet;
import java.util.StringTokenizer;
import java.util.TreeSet;

import org.apache.log4j.Logger;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.util.Assert;

import com.netapp.myasup.core.MyAsupUserDetails;
import com.netapp.myasup.core.UserAgentInfo;

/**
 * Concrete class of User Details
 * @author velur 
 *
 */
public class UserImpl implements MyAsupUserDetails {
	
	Logger log = Logger.getLogger(UserImpl.class);
	
	
	private static final long serialVersionUID = 1L;

	Collection<GrantedAuthority> authorities;
	//this.authorities = Collections.unmodifiableSet(sortAuthorities(authorities));
	
	private String userName;	
	private String password;	
	private boolean accountNonExpired;	
	private boolean accountNonLocked;	
	private boolean credentialsNonExpired;	
	private boolean enabled;
	private String userId;
	private String lastName;
	private String firstName;
	private String email;
	private String crmCustList;
	private String userDn;
	private String companyId;
	private String crmCompanyId;
	private String crmCustomerType;
	private String agreementType;
	private String siteName;
	private String companyName;
	private String vanId;
	private String user;
	private boolean internal;
	private boolean external;
	private boolean reseller;
	private boolean securePerson;
	private UserAgentInfo userAgentInfo;
	private String designation;
	
	public void setInternal(boolean internal) {
		this.internal = internal;
	}

	public void setExternal(boolean external) {
		this.external = external;
	}

	public void setReseller(boolean reseller) {
		this.reseller = reseller;
	}

	public String getCrmCompanyId() {
		return crmCompanyId;
	}

	public void setCrmCompanyId(String crmCompanyId) {
		this.crmCompanyId = crmCompanyId;
	}

	public String getVanId() {
		return vanId;
	}
	
	public void setVanId(String vanId) {
		this.vanId = vanId;
	}

	public void setUserId(String userId) {
		this.userId = userId;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public void setEmail(String email) {
		this.email = email;
	}
	
	public void setSecurePerson(boolean securePerson) {
		this.securePerson = securePerson;		
	}

	public void setCrmCustList(String crmCustList) {
		this.crmCustList = crmCustList;
		/*if (crmCustList != null && crmCustList.length() > 0) {
			StringTokenizer str = new StringTokenizer(crmCustList, ",");
			m_custList = new ArrayList();
			while (str.hasMoreTokens()) {
				m_custList.add(Long.parseLong(str.nextToken()));
			}
		}
		this.setCustomerListAsString(customers);
		LOG.info("****CUSTOMER LIST IS" + m_custList);*/
	}
	
	public void setDesignation(String designation) {
		this.designation = designation;
	}

	public void setUserDn(String userDn) {
		this.userDn = userDn.toLowerCase();
		userDn=userDn.toLowerCase();
		if (userDn != null && userDn.contains("internal")) {
			if (this.getCrmCustomerType() != null && this.getCrmCustomerType().equalsIgnoreCase(MyAsupUserDetails.END_USER)) {
				this.internal = true;
			}
			// THIS HANDLES THE CASE WHEN CRMCUTOMERTYPE IS EMPTY FOR NETAPP
			// INTERNAL USERS
			else if (userDn.contains("netapp.com")) {
				this.internal = true;
			}
		} else if (userDn != null && userDn.contains("external")) {
			if (this.agreementType != null && (this.agreementType.equalsIgnoreCase(MyAsupUserDetails.SP_COMPANY) || this.agreementType.equalsIgnoreCase(MyAsupUserDetails.ASP_COMPANY))) {
				this.reseller = true;
			} else if (this.crmCustomerType != null && this.crmCustomerType.equalsIgnoreCase(MyAsupUserDetails.RESELLER)) {
				this.reseller = true;
			} else if (this.crmCustomerType != null && this.crmCustomerType.equalsIgnoreCase(MyAsupUserDetails.TPMPLUS)) {
				this.reseller = true;
			} else if (this.crmCustomerType != null && this.crmCustomerType.equalsIgnoreCase(MyAsupUserDetails.END_USER)) {
				this.external = true;
			} else {
				this.external = true;
			}
		} else {			
			log.error("UserDn Unknown " + userDn);
		};
	}

	public void setCompanyId(String companyId) {
		this.companyId = companyId;
	}

	public void setCrmCustomerType(String crmCustomerType) {
		this.crmCustomerType = crmCustomerType;
	}

	public void setAgreementType(String agreementType) {
		this.agreementType = agreementType;
	}

	public void setSiteName(String siteName) {
		this.siteName = siteName;
	}

	public void setCompanyName(String companyName) {
		this.companyName = companyName;
	}

	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	public void setAuthorities(Collection<GrantedAuthority> authorities) {
		this.authorities = Collections.unmodifiableSet(sortAuthorities(authorities));
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public void setAccountNonExpired(boolean accountNonExpired) {
		this.accountNonExpired = accountNonExpired;
	}

	public void setAccountNonLocked(boolean accountNonLocked) {
		this.accountNonLocked = accountNonLocked;
	}

	public void setCredentialsNonExpired(boolean credentialsNonExpired) {
		this.credentialsNonExpired = credentialsNonExpired;
	}

	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}

	@Override
	public Collection<GrantedAuthority> getAuthorities() {
		return authorities;
	}

	@Override
	public String getPassword() {
		return password;
	}

	@Override
	public String getUsername() {		
		return userName;
	}

	@Override
	public boolean isAccountNonExpired() {		
		return accountNonExpired;
	}

	@Override
	public boolean isAccountNonLocked() {		
		return accountNonLocked;
	}

	@Override
	public boolean isCredentialsNonExpired() {		
		return credentialsNonExpired;
	}

	@Override
	public boolean isEnabled() {
		return enabled;
	}

	@Override
	public String getUserId() {		
		return userId;
	}

	@Override
	public String getFirstName() {		
		return firstName;
	}

	@Override
	public String getLastName() {		
		return lastName;
	}

	@Override
	public String getEmail() {		
		return email;
	}

	@Override
	public String getCrmCustList() {		
		return crmCustList;
	}

	@Override
	public String getUserDn() {		
		return userDn;
	}

	@Override
	public String getCompanyId() {		
		return companyId;
	}

	@Override
	public String getCrmCustomerType() {		
		return crmCustomerType;
	}

	@Override
	public String getAgreementType() {		
		return agreementType;
	}

	@Override
	public String getSiteName() {		
		return siteName;
	}

	@Override
	public String getCompanyName() {		
		return companyName;
	}

	@Override
	public String getCrmContactId() {		
		return vanId;
	}
	
	@Override 
	public boolean isSecurePerson() {
		return securePerson;
	}
	
	@Override
	public String getDesignation() {
		return designation;
	}	

	/**
	 * @return the userAgentInfo
	 */
	public UserAgentInfo getUserAgentInfo() {
		return userAgentInfo;
	}
	
	public String getUser() {
		return getFirstName()+" "+getLastName();
	}
	
	public void setUser(String user) {
		this.user = user;
	}

	/**
	 * @param userAgentInfo the userAgentInfo to set
	 */
	public void setUserAgentInfo(UserAgentInfo userAgentInfo) {
		this.userAgentInfo = userAgentInfo;
	}

	private static SortedSet<GrantedAuthority> sortAuthorities(Collection<? extends GrantedAuthority> authorities) {
        Assert.notNull(authorities, "Cannot pass a null GrantedAuthority collection");
        // Ensure array iteration order is predictable (as per UserDetails.getAuthorities() contract and SEC-717)
        SortedSet<GrantedAuthority> sortedAuthorities =
            new TreeSet<GrantedAuthority>(new AuthorityComparator());

        for (GrantedAuthority grantedAuthority : authorities) {
            Assert.notNull(grantedAuthority, "GrantedAuthority list cannot contain any null elements");
            sortedAuthorities.add(grantedAuthority);
        }

        return sortedAuthorities;
    }
	
	private static class AuthorityComparator implements Comparator<GrantedAuthority>, Serializable {
        public int compare(GrantedAuthority g1, GrantedAuthority g2) {
            // Neither should ever be null as each entry is checked before adding it to the set.
            // If the authority is null, it is a custom authority and should precede others.
            if (g2.getAuthority() == null) {
                return -1;
            }

            if (g1.getAuthority() == null) {
                return 1;
            }

            return g1.getAuthority().compareTo(g2.getAuthority());
        }
    }

    /**
     * Returns {@code true} if the supplied object is a {@code User} instance with the
     * same {@code username} value.
     * <p>
     * In other words, the objects are equal if they have the same username, representing the
     * same principal.
     */
    @Override
    public boolean equals(Object rhs) {
        if (rhs instanceof User) {
            return userName.equals(((UserImpl) rhs).userName);
        }
        return false;
    }

    /**
     * Returns the hashcode of the {@code username}.
     */
    @Override
    public int hashCode() {
        return userName.hashCode();
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(super.toString()).append(": ");
        sb.append("sm_user: ").append(this.userName).append("; ");
        sb.append("sm_userdn: ").append(this.userDn).append("; ");
        sb.append("crmcustlist : ").append(this.crmCustList).append("; ");
        sb.append("crmcustomertype : ").append(this.crmCustomerType).append("; ");
        sb.append("firstname : ").append(this.firstName).append("; ");
        sb.append("lastname : ").append(this.lastName).append("; ");
        sb.append("agreementtype : ").append(this.agreementType).append("; ");
        sb.append("mail : ").append(this.email).append("; ");
        sb.append("secure_person :").append(this.securePerson).append("; ");
        sb.append("User Type : ");
        if (isInternal()) sb.append("Internal").append("; ");
        if (isExternal()) sb.append("External").append("; ");
        if (isReseller()) sb.append("Reseller").append("; ");
        sb.append(" ").append(this.getUserAgentInfo()).append("; ");
        
        if (!authorities.isEmpty()) {
            sb.append("Granted Authorities: ");

            boolean first = true;
            for (GrantedAuthority auth : authorities) {
                if (!first) {
                    sb.append(",");
                }
                first = false;

                sb.append(auth);
            }
        } else {
            sb.append("Not granted any authorities");
        }

        return sb.toString();
    }

    
   	@Override
	public boolean isInternal() {		
		return internal;
	}

	@Override
	public boolean isReseller() {		
		return reseller;
	}

	@Override
	public boolean isExternal() {		
		return external;
	}
	
	
	public String getUserType(){
		if(isInternal()){
			return "internal";
		}else
		{
			if(isReseller()){
				return "reseller";
			}else
			{
				if(isExternal()){
					return "external";
				}
			}
			
		}
		return null;
	}
		

}
